﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.Cart;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.Gateways;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Mailing;
using Dynamicweb.Rendering;
using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Xml;

/*
 * This information will help you access your AltaPay test gateway:
 * Link to Gateway: https://testgateway.altapaysecure.com
 * User account details:
 * Username: alv@dynamicweb.com
 * Password: 4z8n9Rdq_cPg
*/

namespace Dynamicweb.Ecommerce.CheckoutHandlers.AltaPay
{
    [AddInName("Altapay/Valitor"), AddInDescription("Altapay/Valitor checkout handler"), AddInUseParameterGrouping(true)]
    public class AltaPayCheckoutHandler : CheckoutHandler, IDropDownOptions, IRemoteCapture, IRemotePartialCapture, ISavedCard, IRecurring, IAmountRefund, IFullReturn, IPartialReturn, ICancelOrder
    {
        private string gatewayUrl;
        
        [AddInParameter("Live gateway URL"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;infoText=if not set then used test gateway even if test mode not set")]
        public string GatewayUrl 
        { 
            get
            {
                return TestModeEnabled || string.IsNullOrEmpty(gatewayUrl) ? TestGatewayUrl : gatewayUrl;
            }
            set
            {
                gatewayUrl = value.Trim().Trim('/');
            }
        }

        [AddInParameter("Username"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true; ")]
        public string Username { get; set; }

        [AddInParameter("Password"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true; ")]
        public string Password { get; set; }

        [AddInParameter("Terminal"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(DropDownParameterEditor), "inputClass=inputControl")]
        public string TerminalName { get; set; }

        [AddInParameter("Payment type"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(DropDownParameterEditor), "inputClass=inputControl")]
        public int PaymentType { get; set; }   

        [AddInParameter("Language"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(DropDownParameterEditor), "inputClass=inputControl")]
        public string Language { get; set; } = "en";

        [AddInParameter("Allow saving cards. (Needs to be enabled in the termnal settings as well)"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(YesNoParameterEditor), "inputClass=inputControl")]
        public bool AllowSaveCards { get; set; }      

        [AddInParameter("Test mode"), AddInParameterGroup("Payment gateway settings:"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool TestMode { get; set; } = true;

        [AddInParameter("Error template"), AddInParameterGroup("Template settings:"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/AltaPay/Error")]
        public string ErrorTemplate { get; set; }

        [AddInParameter("Form template"), AddInParameterGroup("Template settings:"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/AltaPay/Form")]
        public string FormTemplate { get; set; }

        [AddInParameter("Success email template"), AddInParameterGroup("Notification success settings:"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/AltaPay/NotifyUser")]
        public string NotifySuccessEmailTemplate { get; set; }

        [AddInParameter("Success email subject"), AddInParameterGroup("Notification success settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifySuccessEmailSubject { get; set; }

        [AddInParameter("Success email sender name"), AddInParameterGroup("Notification success settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifySuccessEmailSenderName { get; set; }

        [AddInParameter("Success email sender email"), AddInParameterGroup("Notification success settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifySuccessEmailSenderEmail { get; set; }

        [AddInParameter("Error email template"), AddInParameterGroup("Notification error settings:"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/AltaPay/NotifyUser")]
        public string NotifyErrorEmailTemplate { get; set; }

        [AddInParameter("Error email subject"), AddInParameterGroup("Notification error settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifyErrorEmailSubject { get; set; }

        [AddInParameter("Error email sender name"), AddInParameterGroup("Notification error settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifyErrorEmailSenderName { get; set; }

        [AddInParameter("Error email sender email"), AddInParameterGroup("Notification error settings:"), AddInParameterEditor(typeof(TextParameterEditor), "NewGUI=true;")]
        public string NotifyErrorEmailSenderEmail { get; set; }

        public string ApiUrl
        {
            get
            {
                return GatewayUrl + MerchantApiPath;
            }
        }

        private string TestGatewayUrl { get; } = "https://testgateway.altapaysecure.com";

        private string MerchantApiPath { get; } = "/merchant/API/";

        private bool TestModeEnabled
        {
            get
            {
                return TestMode || string.IsNullOrWhiteSpace(gatewayUrl) || TestGatewayUrl.Equals(gatewayUrl);
            }
        }        
      
        private AuthType PaymentTypeValue
        {
            get
            {
                return (AuthType)PaymentType;
            }
        }

        public AltaPayCheckoutHandler()
        {
            ServicePointManager.SecurityProtocol = ServicePointManager.SecurityProtocol | SecurityProtocolType.Tls12;
        }

        public Hashtable GetOptions(string dropdownName)
        {
            Hashtable options = new Hashtable();

            if (string.Equals(dropdownName, "Terminal", StringComparison.InvariantCultureIgnoreCase))
            {
                if (!string.IsNullOrWhiteSpace(ApiUrl) && !string.IsNullOrWhiteSpace(Username) && !string.IsNullOrWhiteSpace(Password))
                {
                    try
                    {
                        AltaPayRequest request = CreateRequest();
                        AltaPayResponse response = request.SendRequest("getTerminals");

                        if (response.Result == RequestResult.Success)
                        {
                            XmlNodeList terminals = response.ResponseDocument.SelectNodes($"APIResponse/Body/Terminals/Terminal");
                            foreach (XmlElement terminal in terminals)
                            {
                                var title = terminal.SelectSingleNode("Title").InnerText;
                                var country = terminal.SelectSingleNode("Country").InnerText;
                                options.Add(title, string.Format("({0}) {1}", country, title));
                            }
                        }
                        else
                        {
                            options.Add("", "There is an error with your credentials. " + string.Format("Errorcode: {0} ({1})", response.ErrorCode, response.ErrorMessage));
                        }
                    }                   
                    catch (Exception ex)
                    {
                        LogError(null, ex, "Error in getTerminals.", ex.Message);
                    }
                }
                else
                {
                    options.Add("", "To view the terminals, please type in and save valid credentials.");
                }
            }

            if (string.Equals(dropdownName, "Payment type", StringComparison.InvariantCultureIgnoreCase))
            {
                options.Add(0, "Payment");
                options.Add(1, "PaymentAndCapture");
                options.Add(2, "Subscription");
                options.Add(3, "SubscriptionAndCharge");
                options.Add(4, "SubscriptionAndReserve");
            }

            if (string.Equals(dropdownName, "Language", StringComparison.InvariantCultureIgnoreCase))
            {
                options.Add("cs", "Czech");
                options.Add("da", "Danish");
                options.Add("de", "German");
                options.Add("en", "English");
                options.Add("es", "Spanish");
                options.Add("fi", "Finnish");
                options.Add("fr", "French");
                options.Add("ja", "Japanese");
                options.Add("lt", "Lithuanian");
                options.Add("nl", "Dutch");
                options.Add("no", "Norwegian");
                options.Add("nb", "Norwegian(Bokmål)");
                options.Add("nn", "Norwegian(Nynorsk)");
                options.Add("pl", "Polish");
                options.Add("sv", "Swedish");
                options.Add("th", "Thai");
                options.Add("tr", "Turkish");
                options.Add("zh", "Chinese");
                options.Add("et", "Estonian");
                options.Add("it", "Italian");
                options.Add("pt", "Portugese");
                options.Add("ru", "Russian");
            }

            return options;
        }

        public override string StartCheckout(Order order)
        {
            LogEvent(order, "Checkout started");

            try
            {
                var request = CreateRequest();

                if (!order.DoSaveCardToken && order.SavedCardId > 0)
                {
                    var savedCard = Services.PaymentCard.GetById(order.SavedCardId);
                    if (savedCard != null && !string.IsNullOrWhiteSpace(savedCard.Token))
                    {
                        request.SetParameter("ccToken", savedCard.Token);
                    }
                }

                request.SetParameter("language", Language);

                string typeValue = PaymentTypeValue.ToString();
                typeValue = char.ToLower(typeValue[0]) + typeValue.Substring(1);
                request.SetParameter("type", typeValue);

                request.SetParameter("shop_orderid", order.Id);
                request.SetParameter("amount", order.TotalPrice.ToString(".00", NumberFormatInfo.InvariantInfo));
                request.SetParameter("currency", order.CurrencyCode);
                request.SetParameter("sales_tax", order.Vat.ToString(".00", NumberFormatInfo.InvariantInfo));
                request.SetParameter("terminal", TerminalName);
                request.SetParameter("sales_reconciliation_identifier", Helpers.GetSri(order));

                AddOrderLines(order, request);

                if (order.ShippingFee.Price > 0.00)
                {
                    AddShipmentOrderLine(order, request);
                }

                AddCustomerInformation(order, request);
                AddCallbackParameters(order, request);

                request.SetParameter("cookie", Helpers.GetCookies());

                AltaPayResponse response = request.SendRequest("createPaymentRequest");
                if (response.Result == RequestResult.Error || response.Result == RequestResult.SystemError)
                {
                    LogError(order, "Payment request returned error: " + response.ErrorMessage);
                    order.Errors.Add(response.ErrorMessage);

                    return RenderError(order, $"An error occurred. {response.ErrorMessage}");
                }

                Context.Current.Response.Redirect(response.Url, true);                
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty; //Do Nothing, this is expected
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
                return RenderError(order, ex.Message);
            }

            return string.Empty;
        }

        private AltaPayRequest CreateRequest()
        {
            return new AltaPayRequest
            {
                Configuration = new RequestConfiguration
                {
                    ApiPath = MerchantApiPath,
                    Gateway = GatewayUrl,
                    UserName = Username,
                    Password = Password
                }
            };
        }

        private void AddOrderLines(Order order, AltaPayRequest request)
        {
            string parameterName = "orderLines";

            for (int i = 0; i < order.OrderLines.Count; i++)
            {
                OrderLine orderLine = order.OrderLines[i];

                request.SetArrayField(parameterName, "goodsType", "item", i);
                request.SetArrayField(parameterName, "description", orderLine.ProductName, i);
                request.SetArrayField(parameterName, "itemId", orderLine.Id, i);
                request.SetArrayField(parameterName, "quantity", orderLine.Quantity, i);
                request.SetArrayField(parameterName, "unitPrice", orderLine.UnitPrice.PriceWithoutVAT, i);
                request.SetArrayField(parameterName, "taxAmount", orderLine.Price.VAT, i);
                request.SetArrayField(parameterName, "taxPercent", orderLine.Price.VATPercent, i);
                request.SetArrayField(parameterName, "unitCode", orderLine.UnitId, i);
            }           
        }

        private void AddShipmentOrderLine(Order order, AltaPayRequest request)
        {
            string parameterName = "orderLines";
            int index = order.OrderLines.Count;

            request.SetArrayField(parameterName, "goodsType", "shipment", index);
            request.SetArrayField(parameterName, "description", order.ShippingMethodDescription, index);
            request.SetArrayField(parameterName, "itemId", order.ShippingMethodId, index);
            request.SetArrayField(parameterName, "quantity", 1, index);
            request.SetArrayField(parameterName, "unitPrice", order.ShippingFee.PriceWithoutVAT, index);
            request.SetArrayField(parameterName, "taxAmount", order.ShippingFee.VAT, index);
            request.SetArrayField(parameterName, "taxPercent", order.ShippingFee.VATPercent, index);
            request.SetArrayField(parameterName, "unitCode", "ShipmentOrderLine", index);
        }

        private void AddCustomerInformation(Order order, AltaPayRequest request)
        {
            string parameterName = "customer_info";

            request.SetField(parameterName, "username", order.CustomerAccessUserUserName);
            request.SetField(parameterName, "customer_phone", order.CustomerPhone);
            request.SetField(parameterName, "email", order.CustomerEmail);

            request.SetField(parameterName, "billing_address", order.CustomerAddress);
            request.SetField(parameterName, "billing_city", order.CustomerCity);
            request.SetField(parameterName, "billing_country", order.CustomerCountryCode);
            request.SetField(parameterName, "billing_firstname", order.CustomerFirstName);
            request.SetField(parameterName, "billing_lastname", order.CustomerSurname);
            request.SetField(parameterName, "billing_postal", order.CustomerZip);
            request.SetField(parameterName, "billing_region", order.CustomerRegion);

            request.SetField(parameterName, "shipping_address", order.DeliveryAddress);
            request.SetField(parameterName, "shipping_city", order.DeliveryCity);
            request.SetField(parameterName, "shipping_country", order.DeliveryCountryCode);
            request.SetField(parameterName, "shipping_firstname", order.DeliveryFirstName);
            request.SetField(parameterName, "shipping_lastname", order.DeliverySurname);
            request.SetField(parameterName, "shipping_postal", order.DeliveryZip);
            request.SetField(parameterName, "shipping_region", order.DeliveryRegion);
        }

        private void AddCallbackParameters(Order order, AltaPayRequest request)
        {
            string parameterName = "config";

            request.SetField(parameterName, "callback_fail", Helpers.GetCallbackUrl(order, CallbackType.Failed));
            request.SetField(parameterName, "callback_ok", Helpers.GetCallbackUrl(order, CallbackType.OK));
            request.SetField(parameterName, "callback_open", Helpers.GetCallbackUrl(order, CallbackType.Open));
            request.SetField(parameterName, "callback_notification", Helpers.GetCallbackUrl(order, CallbackType.Notification));

            if (!string.IsNullOrWhiteSpace(FormTemplate))
            {
                request.SetField(parameterName, "callback_form", Helpers.GetCallbackUrl(order, CallbackType.Form));
            }
        }

        public override string Redirect(Order order)
        {
            var callbackValue = Context.Current.Request["Callback"];
            CallbackType callbackType;
            if (!Enum.TryParse(callbackValue, out callbackType))
            {
                throw new ArgumentException("Unknown callback type.");
            }

            var xmlString = Context.Current.Request["xml"];
            switch (callbackType)
            {
                case CallbackType.Form:
                    return !string.IsNullOrWhiteSpace(FormTemplate) ? Render(order, new Template(FormTemplate)) : string.Empty;
                case CallbackType.Notification:
                    LogEvent(order, "Notification callback received: " + xmlString);
                    break;
                case CallbackType.Open:
                    LogEvent(order, "Open callback received: " + xmlString);
                    break;
                case CallbackType.Failed:
                    LogEvent(order, "Failed callback received: " + xmlString);
                    break;
                case CallbackType.OK:
                    LogEvent(order, "OK callback received: " + xmlString);
                    break;
            }

            try
            {
                var xmlDoc = new XmlDocument();
                xmlDoc.LoadXml(xmlString);

                var status = xmlDoc.SelectSingleNode("//Result").InnerText;
                LogEvent(order, "Redirected to checkout. Status: " + status);

                order.SetXmlResponse(xmlString);

                var errorMsg = string.Empty;
                var errorMsgNode = xmlDoc.SelectSingleNode("//ErrorMessage");
                if (errorMsgNode != null)
                {
                    errorMsg = errorMsgNode.InnerText;
                    if (!string.IsNullOrWhiteSpace(errorMsg))
                        order.Errors.Add("ErrorMessage from checkouthandler: " + errorMsg);
                }

                var cardHolderErrorMessage = xmlDoc.SelectSingleNode("//CardHolderErrorMessage");
                if (cardHolderErrorMessage != null && !string.IsNullOrWhiteSpace(cardHolderErrorMessage.InnerText))
                {
                    errorMsg = cardHolderErrorMessage.InnerText;
                    if (!string.IsNullOrWhiteSpace(cardHolderErrorMessage.InnerText))
                        order.Errors.Add("ErrorMessage from checkouthandler: " + cardHolderErrorMessage.InnerText);
                }

                if (callbackType == CallbackType.Notification)
                {
                    NotifyUser(order, status, errorMsg);
                    // return values from notification callbacks are not displayed to the customer
                    return string.Empty;
                }

                if (callbackType == CallbackType.Failed)
                {
                    return RenderError(order, string.IsNullOrWhiteSpace(errorMsg) ? status : errorMsg);
                }

                if (string.Equals(status, "Success", StringComparison.OrdinalIgnoreCase) ||
                    string.Equals(status, "Open", StringComparison.OrdinalIgnoreCase))
                {
                    order.SetXmlResponse(xmlString);
                    order.TransactionNumber = Context.Current.Request["transaction_id"];
                    order.TransactionType = Context.Current.Request["type"];
                    order.TransactionStatus = Context.Current.Request["status"];
                    order.TransactionAmount = double.Parse(Context.Current.Request["amount"], CultureInfo.InvariantCulture);
                    order.TransactionCardType = xmlDoc.SelectSingleNode("//PaymentSchemeName").InnerText;
                    order.TransactionCardNumber = Context.Current.Request["masked_credit_card"];
                    order.TransactionToken = Context.Current.Request["payment_id"];

                    order.CaptureAmount = 0;

                    var cardName = Context.Current.Request["SaveCard"];
                    var cctoken = xmlDoc.SelectSingleNode("//CreditCardToken").InnerText;
                    if (!string.IsNullOrWhiteSpace(cardName) && !string.IsNullOrWhiteSpace(cctoken))
                    {
                        var savedCard = Services.PaymentCard.CreatePaymentCard(order.CustomerAccessUserId, order.PaymentMethodId, order.LanguageId, cardName, order.TransactionCardType, order.TransactionCardNumber, cctoken);
                        order.SavedCardId = savedCard.ID;
                    }

                    if (string.Equals(status, "Success", StringComparison.OrdinalIgnoreCase))
                    {
                        if (PaymentTypeValue == AuthType.SubscriptionAndCharge)
                        {
                            order.CaptureAmount = double.Parse(xmlDoc.SelectSingleNode("//Transaction[AuthType='subscription_payment']/CapturedAmount").InnerText, CultureInfo.InvariantCulture);
                        }
                        else if (PaymentTypeValue == AuthType.PaymentAndCapture)
                        {
                            order.CaptureAmount = double.Parse(xmlDoc.SelectSingleNode("//CapturedAmount").InnerText, CultureInfo.InvariantCulture);
                        }

                        if (string.Equals(Context.Current.Request["require_capture"], "true", StringComparison.OrdinalIgnoreCase))
                        {
                            order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.NotCaptured, "Payment needs to be captured.");
                        }
                        else
                        {
                            if (order.CaptureAmount == order.TransactionAmount)
                            {
                                LogEvent(order, "Full capture order", DebuggingInfoType.CaptureResult);
                                order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, "Autocapture successful.");
                            }
                            else if (order.CaptureAmount > 0)
                            {
                                LogEvent(order, "Split capture order", DebuggingInfoType.CaptureResult);
                                order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Split, "Partial capture successful.");
                            }
                        }

                        SetOrderComplete(order);
                        CheckoutDone(order);
                        RedirectToCart(order);
                    }
                    else if (string.Equals(status, "Open", StringComparison.OrdinalIgnoreCase))
                    {
                        LogEvent(order, "Altapay has opened payment");

                        SetOrderComplete(order);
                        CheckoutDone(order);
                        RedirectToCart(order);
                    }
                }

                return RenderError(order, xmlDoc.SelectSingleNode("//Status").InnerText);
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty; //Do Nothing, this is expected
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
                return RenderError(order, ex.Message);
            }
        }

        private void NotifyUser(Order order, string status, string errorMsg = "")
        {
            LogEvent(order, string.Format("Sending notification ({0}) email. ErrorMsg: {1}", status, errorMsg));
            try
            {
                Template mailTemplate = null;
                var isSuccess = string.Equals(status, "Success", StringComparison.InvariantCultureIgnoreCase);
                if (isSuccess)
                {
                    if (!string.IsNullOrWhiteSpace(NotifySuccessEmailTemplate))
                        mailTemplate = new Template(NotifySuccessEmailTemplate);
                }
                else
                {
                    if (!string.IsNullOrWhiteSpace(NotifyErrorEmailTemplate))
                        mailTemplate = new Template(NotifyErrorEmailTemplate);
                }
                if (mailTemplate == null)
                {
                    LogError(order, string.Format("Can not send email due to missing '{0}' status template", status));
                    return;
                }

                mailTemplate.SetTag("Order.ID", order.Id);
                mailTemplate.SetTag("Order.Date", order.CompletedDate ?? order.Date);
                mailTemplate.SetTag("Order.TotalPriceFormatted", order.TotalPriceFormatted);
                mailTemplate.SetTag("Order.TotalPrice", order.TotalPrice);
                mailTemplate.SetTag("User.UserName", order.CustomerAccessUserUserName);
                mailTemplate.SetTag("User.Name", order.CustomerName);
                mailTemplate.SetTag("User.Company", order.CustomerCompany);
                mailTemplate.SetTag("User.Address", order.CustomerAddress);
                mailTemplate.SetTag("User.Address2", order.CustomerAddress2);
                mailTemplate.SetTag("User.Zip", order.CustomerZip);
                mailTemplate.SetTag("User.City", order.CustomerCity);
                mailTemplate.SetTag("User.Country", order.CustomerCountry);
                mailTemplate.SetTag("User.Phone", order.CustomerPhone);
                mailTemplate.SetTag("User.Email", order.CustomerEmail);

                using (var mailMessage = new MailMessage())
                {
                    mailMessage.Subject = isSuccess ? NotifySuccessEmailSubject : NotifyErrorEmailSubject;

                    string senderEmail = isSuccess ? NotifySuccessEmailSenderEmail : NotifyErrorEmailSenderEmail;
                    string senderName = isSuccess ? NotifySuccessEmailSenderName : NotifyErrorEmailSenderName;
                    mailMessage.From = new MailAddress(senderEmail, senderName);

                    mailMessage.To.Add(order.CustomerEmail);
                    mailMessage.IsBodyHtml = true;
                    mailMessage.Body = mailTemplate.Output();
                    mailMessage.BodyEncoding = Encoding.UTF8;
                    mailMessage.SubjectEncoding = Encoding.UTF8;
                    mailMessage.HeadersEncoding = Encoding.UTF8;

                    EmailHandler.Send(mailMessage);
                }
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception on Notify User.");
            }
        }

        private string RenderError(Order order, string message)
        {
            LogEvent(order, "Printing error template");

            order.TransactionAmount = 0;
            order.TransactionStatus = "Failed";
            order.Errors.Add(message);
            Services.Orders.Save(order);
            Services.Orders.DowngradeToCart(order);
            order.CartV2StepIndex = 0;
            order.TransactionStatus = string.Empty;
            Common.Context.SetCart(order);

            if (string.IsNullOrWhiteSpace(ErrorTemplate))
            {
                RedirectToCart(order);
            }

            var errorTemplate = new Template(ErrorTemplate);
            errorTemplate.SetTag("CheckoutHandler:ErrorMessage", message);
            return Render(order, errorTemplate);
        }

        #region IRemoteCapture

        public bool CaptureSupported(Order order)
        {
            return PaymentTypeValue == AuthType.Payment
                || PaymentTypeValue == AuthType.Subscription
                || PaymentTypeValue == AuthType.SubscriptionAndReserve;
        }

        public bool SplitCaptureSupported(Order order)
        {
            var xmlDoc = order.GetXmlResponse();
            if (xmlDoc == null)
                return false;

            var supportsCapture = xmlDoc.SelectSingleNode("//SupportsMultipleCaptures");
            return supportsCapture != null && bool.Parse(supportsCapture.InnerText);
        }

        public OrderCaptureInfo Capture(Order order)
        {
            return Capture(order, order.Price.PricePIP, true);
        }

        public OrderCaptureInfo Capture(Order order, long amount, bool final)
        {
            LogEvent(order, "Attempting capture.", DebuggingInfoType.CaptureResult);

            if (order == null)
            {
                LogError(null, "Order not set");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order not set");
            }
            else if (string.IsNullOrEmpty(order.Id))
            {
                LogError(null, "Order id not set");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order id not set");
            }
            else if (string.IsNullOrEmpty(order.TransactionNumber))
            {
                LogError(null, "Transaction number not set");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Transaction number not set");
            }
            else if (amount > order.Price.PricePIP)
            {
                LogError(null, "Amount to capture should be less or equal to order total");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Amount to capture should be less or equal to order total");
            }

            var captureInfo = order.CaptureInfo;

            try
            {
                var request = CreateRequest();

                float amountValue = amount / 100f;
                request.SetParameter("amount", amountValue.ToString(".00", NumberFormatInfo.InvariantInfo));

                request.SetParameter("transaction_id", order.TransactionNumber);
                request.SetParameter("reconciliation_identifier", Helpers.GetSri(order));

                AltaPayResponse response = request.SendRequest("captureReservation");
                LogEvent(order, "Received response from capture request to AltaPay: " + response.CardHolderErrorMessage, DebuggingInfoType.CaptureResult);
                               
                order.TransactionType = response.ReadFromBody("Transactions/Transaction/AuthType");
                order.TransactionStatus = response.ReadFromBody("Transactions/Transaction/TransactionStatus");

                string createdDate = response.ReadFromBody("Transactions/Transaction/CreatedDate");
                order.CaptureInfo.CaptureTime = DateTime.ParseExact(createdDate, "yyyy-MM-dd HH:mm:ss", CultureInfo.InvariantCulture);

                order.CaptureInfo.Message = response.CardHolderErrorMessage;
                if (response.Result == RequestResult.Success)
                {
                    if (amount == order.Price.PricePIP || final)
                    {
                        order.CaptureInfo.State = OrderCaptureInfo.OrderCaptureState.Success;
                    }
                    else
                    {
                        order.CaptureInfo.State = OrderCaptureInfo.OrderCaptureState.Split;
                    }
                }
                else
                {
                    order.CaptureInfo.State = OrderCaptureInfo.OrderCaptureState.Failed;
                }
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message, DebuggingInfoType.CaptureResult);
                captureInfo.Message = "Error during capture: " + ex.Message;
                captureInfo.State = OrderCaptureInfo.OrderCaptureState.Failed;
            }

            Services.Orders.Save(order);

            return captureInfo;
        }
        #endregion

        #region ISavedCard

        public void DeleteSavedCard(int savedCardId)
        {
            var savedCard = Services.PaymentCard.GetById(savedCardId);
            if (savedCard != null)
            {
                savedCard.Delete(savedCard.ID);
            }
        }

        public string UseSavedCard(Order order)
        {
            return StartCheckout(order);
        }

        public bool SavedCardSupported(Order order)
        {
            return AllowSaveCards || SaveCardIsMandatory();
        }

        public bool SaveCardIsMandatory()
        {
            return false;
        }

        #endregion

        #region IRecurring

        public void Recurring(Order order, Order originalOrder)
        {
            LogEvent(originalOrder, "Attempting recurring capture.");
            if (order == null) return;

            try
            {
                var request = CreateRequest();
                request.SetParameter("transaction_id", originalOrder.TransactionNumber);
                request.SetParameter("amount", order.TotalPrice.ToString(".00", NumberFormatInfo.InvariantInfo));

                AltaPayResponse response = request.SendRequest("chargeSubscription");

                XmlNodeList transactions = response.ResponseDocument.SelectNodes($"APIResponse/Body/Transactions/Transaction");
                XmlNode payment = transactions?.Item(0);
                XmlNode recurringPayment = transactions?.Item(1);

                order.TransactionNumber = payment?.SelectSingleNode("TransactionId")?.InnerText ?? string.Empty;
                order.TransactionType = "ChargeSubscription";
                order.TransactionStatus = recurringPayment?.SelectSingleNode("TransactionStatus")?.InnerText ?? string.Empty;
                order.TransactionAmount = order.TotalPrice;
                order.TransactionCardType = "Unknown";
                order.TransactionCardNumber = recurringPayment?.SelectSingleNode("CreditCardMaskedPan")?.InnerText ?? string.Empty;
                order.TransactionToken = recurringPayment?.SelectSingleNode("PaymentId")?.InnerText ?? string.Empty;

                if (response.Result == RequestResult.Success)
                {
                    LogEvent(order, "Recurring succeeded: " + response.CardHolderErrorMessage);

                    order.CaptureAmount = Converter.ToDouble(recurringPayment?.SelectSingleNode("CapturedAmount")?.InnerText);
                    order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, "Recurring payment captured.");
                    order.CaptureInfo.CaptureTime = DateTime.Now;

                    SetOrderComplete(order);
                    CheckoutDone(order);
                }
                else
                {
                    order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Recurring payment failed.");
                    LogEvent(order, "Recurring order failed for {0} (based on {1}). The payment failed with the message: {2}", DebuggingInfoType.RecurringError, order.Id, originalOrder.Id, response.ErrorMessage);
                }
            }
            catch (Exception ex)
            {
                LogError(order, "Unhandled exception. Recurring order failed for {0} (based on {1}). The payment failed with the message: {2}", DebuggingInfoType.RecurringError, order.Id, originalOrder.Id, ex.Message);
            }
        }

        public bool RecurringSupported(Order order)
        {
            return TerminalName.IndexOf("iDEAL", StringComparison.OrdinalIgnoreCase) == -1 &&
                    (PaymentTypeValue == AuthType.Subscription ||
                    PaymentTypeValue == AuthType.SubscriptionAndCharge ||
                    PaymentTypeValue == AuthType.SubscriptionAndReserve);
        }

        #endregion

        #region IAmountRefunt, IPartialReturn, IFullReturn

        public void FullReturn(Order order)
        {
            AmountRefund(order, order);
        }

        public void PartialReturn(Order order, Order originalOrder)
        {
            AmountRefund(order, originalOrder);
        }

        public void AmountRefund(Order order, Order originalOrder)
        {
            AmountRefundResult(order, originalOrder);
        }

        /// <summary>
        /// amount refund with a boolean result
        /// </summary>
        /// <param name="order"></param>
        /// <param name="originalOrder"></param>
        /// <returns><c>true</c> if return operation succeeded</returns>
        private bool AmountRefundResult(Order order, Order originalOrder)
        {
            LogEvent(originalOrder, "Refund started");

            var returnAmount = order.TotalPrice > originalOrder.CaptureAmount ? originalOrder.CaptureAmount : order.TotalPrice;

            try
            {
                var xmlDoc = originalOrder.GetXmlResponse();

                if (originalOrder.CaptureInfo.State == OrderCaptureInfo.OrderCaptureState.Success || originalOrder.CaptureInfo.State == OrderCaptureInfo.OrderCaptureState.Split)
                {
                    var supportsRefunds = xmlDoc.SelectSingleNode("//SupportsRefunds");
                    if (supportsRefunds != null && !string.Equals(supportsRefunds.InnerText, "true", StringComparison.InvariantCultureIgnoreCase))
                    {
                        OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.Failed, "Order does not support refunds.", 0, originalOrder);
                        LogEvent(originalOrder, "Order does not support refunds.");
                        return false;
                    }

                    var fullReturnOperation = originalOrder.ReturnOperations.FirstOrDefault(returnOperation => returnOperation.State == OrderReturnOperationState.FullyReturned);
                    var returned = originalOrder.ReturnOperations.Where(returnOperation => returnOperation.State == OrderReturnOperationState.PartiallyReturned || returnOperation.State == OrderReturnOperationState.FullyReturned).Sum(x => x.Amount);
                    if (fullReturnOperation != null || (originalOrder.CaptureAmount - returned) < 0.01)
                    {
                        OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.Failed, "Order already returned", returnAmount, originalOrder);
                        LogError(originalOrder, "AltaPay payment refund operation failed. Order already returned.");
                        return false;
                    }

                    var request = CreateRequest();  //new AlpaPayApi.Service.RefundRequest();
                    request.SetParameter("amount", returnAmount.ToString(".00", NumberFormatInfo.InvariantInfo));

                    request.SetParameter("transaction_id", originalOrder.TransactionNumber);
                    AddRefundOrderLine(order, originalOrder, request);

                    var response = request.SendRequest("refundCapturedReservation");
                    bool paymentIsEmpty = string.IsNullOrEmpty(response.ReadFromBody("Transactions/Transaction"));

                    if (response.Result != RequestResult.Success || paymentIsEmpty)
                    {
                        var message = $"Altapay response not valid. Response message - {response.CardHolderErrorMessage}";
                        OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.Failed, message, returnAmount, originalOrder);
                        LogEvent(originalOrder, message);

                        return false;
                    }

                    originalOrder.TransactionNumber = response.ReadFromBody("Transactions/Transaction/TransactionId");
                    originalOrder.TransactionType = "Refund";
                    originalOrder.TransactionStatus = response.ReadFromBody("Transactions/Transaction/TransactionStatus");
                    originalOrder.TransactionAmount = Converter.ToDouble(response.ReadFromBody("Transactions/Transaction/RefundedAmount"));
                    originalOrder.TransactionCardType = "Unknown";
                    originalOrder.TransactionCardNumber = response.ReadFromBody("Transactions/Transaction/CreditCardMaskedPan");
                    originalOrder.TransactionToken = response.ReadFromBody("Transactions/Transaction/PaymentId");

                    if (originalOrder.CaptureAmount - (returnAmount + returned) < 0.01)
                    {
                        OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.FullyReturned, "AltaPay has full refunded payment.", returnAmount, originalOrder);
                    }
                    else
                    {
                        OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.PartiallyReturned, "AltaPay has partial refunded payment.", returnAmount, originalOrder);
                    }

                    LogEvent(originalOrder, "ResultMessage: " + response.CardHolderErrorMessage);

                    return response.Result == RequestResult.Success;
                }
                else
                {
                    var message = "Can't refund a non-captured order.";
                    OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.Failed, message, returnAmount, originalOrder);
                    LogEvent(originalOrder, message);

                    return false;
                }
            }
            catch (Exception ex)
            {
                OrderReturnInfo.SaveReturnOperation(OrderReturnOperationState.Failed, $"Failed return operation with exception: {ex.Message}", returnAmount, originalOrder);
                LogError(originalOrder, ex, "Unhandled exception with message: {0}", ex.Message);

                return false;
            }
        }

        private void AddRefundOrderLine(Order order, Order originalOrder, AltaPayRequest request)
        {
            string parameterName = "orderLines";
            int index = 1;

            request.SetArrayField(parameterName, "goodsType", "refund", index);
            request.SetArrayField(parameterName, "description", $"{originalOrder.Id} refund {order.Price.PriceWithVAT}", index);
            request.SetArrayField(parameterName, "itemId", originalOrder.Id, index);
            request.SetArrayField(parameterName, "quantity", 1, index);
            request.SetArrayField(parameterName, "unitPrice", order.Price.PriceWithoutVAT, index);
            request.SetArrayField(parameterName, "taxAmount", order.Price.VAT, index);
            request.SetArrayField(parameterName, "taxPercent", order.Price.VATPercent, index);
            request.SetArrayField(parameterName, "unitCode", originalOrder.Id, index);
        }

        #endregion

        #region ICancelOrder

        public bool CancelOrder(Order order)
        {
            try
            {
                if (order.CaptureInfo != null && (order.CaptureInfo.State == OrderCaptureInfo.OrderCaptureState.Success || order.CaptureInfo.State == OrderCaptureInfo.OrderCaptureState.Split))
                {
                    return AmountRefundResult(order, order);
                }

                LogEvent(order, "Cancelling order. Attempting to release reserved amounts.");
                var xmlDoc = order.GetXmlResponse();
                if (!string.Equals(xmlDoc.SelectSingleNode("//SupportsRelease").InnerText, "true", StringComparison.InvariantCultureIgnoreCase))
                {
                    LogEvent(order, "Order does not support releases.");
                    return false;
                }

                var request = CreateRequest();
                request.SetParameter("transaction_id", order.TransactionNumber);
                var response = request.SendRequest("releaseReservation");

                bool paymentIsEmpty = string.IsNullOrEmpty(response.ReadFromBody("Transactions/Transaction"));

                if (response.Result != RequestResult.Success || paymentIsEmpty)
                {
                    LogEvent(order, $"Altapay release request failed. Response message - {response.CardHolderErrorMessage}");

                    return false;
                }

                order.TransactionNumber = response.ReadFromBody("Transactions/Transaction/TransactionId");
                order.TransactionType = "Release";
                order.TransactionStatus = response.ReadFromBody("Transactions/Transaction/TransactionStatus");
                order.TransactionAmount = order.TotalPrice;
                order.TransactionCardType = "Unknown";
                order.TransactionCardNumber = response.ReadFromBody("Transactions/Transaction/CreditCardMaskedPan");
                order.TransactionToken = response.ReadFromBody("Transactions/Transaction/PaymentId");
                Services.Orders.Save(order);

                LogEvent(order, "Cancel order status: '" + response.Result + "'. ResultMessage: " + response.CardHolderErrorMessage);

                return response.Result == RequestResult.Success;
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);

                return false;
            }
        }

        #endregion
    }
}