﻿using Dynamicweb.Core;
using System;
using System.Xml;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.AltaPay
{
    /// <summary>
    /// The server response presented as wrapper around xml-document
    /// </summary>
    internal class AltaPayResponse
    {
        /// <summary>
        /// The server response as xml document
        /// </summary>
        public XmlDocument ResponseDocument { get; set; }

        /// <summary>
        /// Result of request. If this is equal to None, then probably there wasn't Body node in the response.
        /// </summary>
        public RequestResult Result { get; set; } = RequestResult.None;

        /// <summary>
        /// Url to redirect user
        /// </summary>
        public string Url { get; set; }

        /// <summary>
        /// Error code
        /// </summary>
        public int ErrorCode { get; set; }

        /// <summary>
        /// Error message
        /// </summary>
        public string ErrorMessage { get; set; }

        /// <summary>
        /// Additional error information
        /// </summary>
        public string CardHolderErrorMessage { get; set; }

        /// <summary>
        /// An error message for the merchant's eyes only, but provides more detail as to what went wrong
        /// </summary>
        public string MerchantErrorMessage { get; set; }

        public AltaPayResponse(XmlDocument response)
        {
            ResponseDocument = response;

            ErrorCode = Converter.ToInt32(ReadFromHeader("ErrorCode"));
            ErrorMessage = ReadFromHeader("ErrorMessage");

            RequestResult result;
            if (Enum.TryParse(ReadFromBody("Result"), out result))
            {
                Result = result;
            }
            else if (ErrorCode > 0)
            {
                Result = RequestResult.SystemError;
            }
           
            CardHolderErrorMessage = ReadFromBody("CardHolderErrorMessage");
            MerchantErrorMessage = ReadFromBody("MerchantErrorMessage");
            Url = ReadFromBody("Url");
        }

        /// <summary>
        /// Reads node text by path from Header response node. It will found element by next path APIResponse/Header/{your path}
        /// </summary>
        /// <param name="path">The path</param>
        /// <returns>Inner text of node or empty string</returns>
        public string ReadFromHeader(string path)
        {
            return ResponseDocument.SelectSingleNode($"APIResponse/Header/{path}")?.InnerText ?? string.Empty;
        }

        /// <summary>
        /// Reads node text by path from Body response node. It will found element by next path APIResponse/Body/{your path}
        /// </summary>
        /// <param name="path">The path</param>
        /// <returns>Inner text of node or empty string</returns>
        public string ReadFromBody(string path)
        {
            return ResponseDocument.SelectSingleNode($"APIResponse/Body/{path}")?.InnerText ?? string.Empty;
        }        
    }
}
