﻿using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Environment;
using Dynamicweb.Environment.Helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.AltaPay
{
    internal static class Helpers
    {
        /// <summary> 
        /// Gets a SalesReconciliationIdentifier based on the order shopname and id
        /// </summary>
        /// <param name="order">The current Order</param>
        /// <returns>SalesReconciliationIdentifier</returns>
        public static string GetSri(Order order)
        {
            return string.Format("Order-{1}_", order.Id, DateTime.Now.ToString("yyyyMMddHHmmssffff"));
        }

        /// <summary>
        /// GetCallbackUrl with parameters required by the checkouthandler
        /// </summary>
        /// <param name="order">The order</param>
        /// <param name="callback">Callback type</param>
        /// <returns>URL with additional parameters</returns>
        public static string GetCallbackUrl(Order order, CallbackType callback)
        {
            string callbackName = Enum.GetName(typeof(CallbackType), callback);
            string queryString = LinkHelper.AddToQueryString(Context.Current.Request.Url.Query, $"Callback={callbackName}");

            if (callback == CallbackType.OK || callback == CallbackType.Open)
            {
                string cardName = string.Empty;
                if (order.DoSaveCardToken)
                {
                    cardName = !string.IsNullOrWhiteSpace(order.SavedCardDraftName) ? order.SavedCardDraftName : order.Id;
                }

                queryString = LinkHelper.AddToQueryString(queryString, $"SaveCard={cardName}");
            }

            string pathAndQueryString = LinkHelper.ReplaceQueryString(queryString, "CheckoutHandlerOrderID", order.Id);

            return $"{Context.Current.Request.Url.Scheme}://{Context.Current.Request.Url.Host}{pathAndQueryString}";
        }

        /// <summary>
        /// Get a combined string of all the cookies
        /// </summary>
        public static string GetCookies()
        {
            ICookieCollection cookies = Context.Current.Request.Cookies;
            if (cookies is null || cookies.Count == 0) return string.Empty;

            var cookieStringCollection = new List<string>();

            foreach (string cookieName in cookies)
            {
                cookieStringCollection.Add(string.Format("{0}={1}", cookieName, cookies[cookieName].Value));
            }

            return string.Join(";", cookieStringCollection);
        }                      

        public const string XmlResponseOrderFieldName = "AltaPayXml";

        /// <summary>
        /// Get AltaPay's xml response
        /// </summary>
        /// <param name="order"></param>
        /// <returns></returns>
        public static XmlDocument GetXmlResponse(this Order order)
        {
            var xmlResponseFieldValue = order.OrderFieldValues.FirstOrDefault(x => x.OrderField.SystemName == XmlResponseOrderFieldName);
            if (xmlResponseFieldValue == null || string.IsNullOrWhiteSpace((string)xmlResponseFieldValue.Value))
                return null;

            var xmlDocument = new XmlDocument();
            xmlDocument.LoadXml((string)xmlResponseFieldValue.Value);
            return xmlDocument;
        }

        /// <summary>
        /// Set AltaPay's XML response string as a custom order field. Creates a new orderfield if needed.
        /// </summary>
        /// <param name="order"></param>
        /// <param name="xmlString"></param>
        public static void SetXmlResponse(this Order order, string xmlString)
        {
            var xmlResponseFieldValue = order.OrderFieldValues.FirstOrDefault(x => x.OrderField.SystemName == XmlResponseOrderFieldName);
            if (xmlResponseFieldValue == null)
            {
                var xmlResponseOrderField = new OrderField(XmlResponseOrderFieldName)
                {
                    Name = XmlResponseOrderFieldName,
                    SystemName = XmlResponseOrderFieldName,
                    TemplateName = XmlResponseOrderFieldName,
                    TypeId = 2 //long text
                };
                xmlResponseOrderField.Save(XmlResponseOrderFieldName);
                var x = new OrderFieldValue();
                order.OrderFieldValues.Add(new OrderFieldValue(xmlResponseOrderField, xmlString));
            }
            else
            {
                xmlResponseFieldValue.Value = xmlString;
            }
        }
       
    }
}