﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.Cart;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.Gateways;
using Dynamicweb.Environment;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Rendering;
using Dynamicweb.SystemTools;
using System;
using System.Collections;
using System.IO;
using System.Xml.Serialization;
using BBS = Dynamicweb.Ecommerce.CheckoutHandlers.BBSPayment.no.bbs.epayment;
using BBSTest = Dynamicweb.Ecommerce.CheckoutHandlers.BBSPayment.test.epayment.nets.eu;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.BBSPayment
{
    /// <summary>
    /// BBSPayment Checkout Handler½
    /// </summary>
    [
        AddInName("Nets Netaxept"),
        AddInDescription("epayment.bbs.no Supported currency: NOK")
    ]
    public class BBSPayment : CheckoutHandler, IRemoteCapture, IDropDownOptions
    {
        #region Addin parameters

        [AddInParameter("Merchant ID"), AddInParameterEditor(typeof(TextParameterEditor), ""), AddInDescription("Required to generate Merchant Signature.")]
        public string Merchant { get; set; }

        [AddInParameter("Merchant token key"), AddInParameterEditor(typeof(TextParameterEditor), ""), AddInDescription("Required to generate Merchant Signature.")]
        public string Token { get; set; }

        [AddInParameter("The type of operation"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true; none=false; SortBy=Value;")]
        public string OperationType { get; set; }

        /// <summary>
        /// if not set - using current frontend culture if it is supported by payment - if not system will use it's own default "no-NO"
        /// </summary>
        [AddInParameter("Terminal language"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true; none=false; SortBy=Key;")]
        public string TerminalLanguage { get; set; }

        [AddInParameter("Cancel template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/BBS/Cancel")]
        public string CancelTemplate { get; set; }

        [AddInParameter("Error template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/BBS/Error")]
        public string ErrorTemplate { get; set; }

        [AddInParameter("Test"), AddInParameterEditor(typeof(YesNoParameterEditor), ""), AddInDescription("")]
        public bool Test { get; set; }

        #endregion

        #region StartCheckout

        /// <summary>
        /// Starts order checkout procedure
        /// </summary>
        /// <param name="order">Order to be checked out</param>
        /// <returns>String representation of template output</returns>
        public override string StartCheckout(Order order)
        {
            var bbsUrl = string.Empty;
            var bbsResponse = string.Empty;

            try
            {
                LogEvent(order, "Checkout started");

                if (Test)
                {
                    var client = new BBSTest.NetaxeptClient();
                    var request = new BBSTest.RegisterRequest();
                    InitTestRegisterRequest(request, order);

                    BBSTest.RegisterResponse response = client.Register(Merchant, Token, request);
                    bbsResponse = response.TransactionId;
                    bbsUrl = "https://test.epayment.nets.eu/terminal/default.aspx?merchantId={0}&transactionId={1}";
                }
                else
                {
                    var client = new BBS.NetaxeptClient();
                    var request = new BBS.RegisterRequest();
                    InitProductionregisterRequest(request, order);

                    BBS.RegisterResponse response = client.Register(Merchant, Token, request);
                    bbsResponse = response.TransactionId;
                    bbsUrl = "https://epayment.bbs.no/terminal/default.aspx?merchantId={0}&transactionId={1}";
                }

                //if (order.CurrencyCode != "NOK")
                //{
                //	throw new Exception(Dynamicweb.SystemTools.Translate.Translate("Only NOK(Norwegian Kroner) currency is allowed. Order currency: %%", "%%", order.CurrencyCode));
                //}

                Context.Current.Response.Redirect(string.Format(bbsUrl, Merchant, bbsResponse));
                return string.Empty;
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order, ex.Message);
            }
        }

        private void InitProductionregisterRequest(BBS.RegisterRequest request, Order order)
        {

            BBS.Terminal terminal = new BBS.Terminal
            {
                RedirectUrl = BaseUrl(order),
                Language = GetLanguage()
            };

            BBS.Order bbsOrder = new BBS.Order
            {
                Amount = order.Price.PricePIP.ToString(),
                CurrencyCode = order.CurrencyCode,
                OrderNumber = order.Id
            };

            BBS.Environment environment = new BBS.Environment
            {
                WebServicePlatform = "WCF"
            };

            string customerName = Converter.ToString(order.CustomerName);
            int delPos = customerName.IndexOf(' ');
            string firstName = delPos >= 0 ? customerName.Substring(0, delPos) : customerName;
            string lastName = delPos >= 0 ? customerName.Substring(delPos + 1) : string.Empty;

            BBS.Customer customer = new BBS.Customer
            {
                FirstName = firstName,
                LastName = lastName,
                Email = order.CustomerEmail,
                Country = order.CustomerCountryCode
            };

            request.Order = bbsOrder;
            request.Terminal = terminal;
            request.Environment = environment;
            request.Customer = customer;
        }

        private void InitTestRegisterRequest(BBSTest.RegisterRequest request, Order order)
        {
            BBSTest.Terminal terminal = new BBSTest.Terminal
            {
                RedirectUrl = BaseUrl(order),
                Language = GetLanguage()
            };

            BBSTest.Order bbsOrder = new BBSTest.Order
            {
                Amount = order.Price.PricePIP.ToString(),
                CurrencyCode = order.CurrencyCode,
                OrderNumber = order.Id
            };

            BBSTest.Environment environment = new BBSTest.Environment
            {
                WebServicePlatform = "WCF"
            };

            string customerName = Converter.ToString(order.CustomerName);
            int delPos = customerName.IndexOf(' ');
            string firstName = delPos >= 0 ? customerName.Substring(0, delPos) : customerName;
            string lastName = delPos >= 0 ? customerName.Substring(delPos + 1) : string.Empty;

            BBSTest.Customer customer = new BBSTest.Customer
            {
                FirstName = firstName,
                LastName = lastName,
                Email = order.CustomerEmail,
                Country = order.CustomerCountryCode
            };

            request.Order = bbsOrder;
            request.Terminal = terminal;
            request.Environment = environment;
            request.Customer = customer;
        }

        #endregion

        #region Form properties

        private static string BaseUrl(Order order)
        {
            var disablePortNumber = Dynamicweb.Configuration.SystemConfiguration.Instance.GetValue("/Globalsettings/System/http/DisableBaseHrefPort") == "True";
            var portString = (Context.Current.Request.Url.IsDefaultPort || disablePortNumber) ? string.Empty : string.Format(":{0}", Context.Current.Request.Url.Port);
            var pageId = Dynamicweb.Context.Current.Request["ID"] == null ? string.Empty : string.Format("ID={0}&", Dynamicweb.Context.Current.Request["ID"]);

            return string.Format(
                "{0}://{1}{2}/Default.aspx?{3}{4}={5}",
                Context.Current.Request.Url.Scheme, Context.Current.Request.Url.Host, portString, pageId, OrderIdRequestName, order.Id
            );
        }

        #endregion

        #region Redirect 

        /// <summary>
        /// Redirects user to payment step or to canceled order step
        /// </summary>
        /// <param name="order">Order for processing</param>
        /// <returns>String representation of template output</returns>
        public override string Redirect(Order order)
        {
            try
            {
                LogEvent(null, "Redirected to BBS CheckoutHandler");

                string responseCode = Dynamicweb.Context.Current.Request["responseCode"];
                string transactionID = Dynamicweb.Context.Current.Request["transactionId"];

                if (string.IsNullOrEmpty(responseCode))
                {
                    LogError(order, "The request variable 'responseCode' is not present");
                    return "";
                }

                if (string.IsNullOrEmpty(transactionID))
                {
                    LogError(order, "The request variable 'transactionId' is not present");
                    return string.Empty;
                }

                switch (responseCode)
                {
                    case "OK":
                        return StateOk(order, transactionID);

                    case "Cancel":
                        return StateCancel(order);

                    default:
                        return StateDecline(order, transactionID);
                }
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order, ex.Message);
            }
        }

        private string StateOk(Order order, string transactionId)
        {
            LogEvent(order, "State ok");

            ProcessPaymentOperation(order, transactionId);

            if (order.Complete)
            {
                RedirectToCart(order);
                return null;
            }
            else
            {
                if (Common.Context.Cart == null)
                {
                    CheckoutDone(order);
                }

                LogError(order, "Called State ok, but order is not set complete.");

                return PrintErrorTemplate(order, "Called State ok, but order is not set complete.");
            }
        }

        private string StateCancel(Order order)
        {
            LogEvent(order, "State cancel");

            order.TransactionStatus = "Cancelled";
            Services.Orders.Save(order);

            CheckoutDone(order);

            var cancelTemplate = new Template(CancelTemplate);
            var orderRenderer = new Ecommerce.Frontend.Renderer();
            orderRenderer.RenderOrderDetails(cancelTemplate, order, true);

            return cancelTemplate.Output();
        }

        private string StateDecline(Order order, string transactionId)
        {
            string declineReason;
            var clientTest = new BBSTest.NetaxeptClient();
            var client = new BBS.NetaxeptClient();

            if (Test)
            {
                BBSTest.QueryRequest requestTest = new BBSTest.QueryRequest
                {
                    TransactionId = transactionId
                };
                BBSTest.QueryResponse responseTest = clientTest.Query(Merchant, Token, requestTest);
                BBSTest.PaymentInfo paymentInfoTest = (BBSTest.PaymentInfo)responseTest;
                declineReason = string.Format("{0}/{1}: {2}", paymentInfoTest.Error.ResponseCode, paymentInfoTest.Error.ResponseSource, paymentInfoTest.Error.ResponseText);
            }
            else
            {
                BBS.QueryRequest request = new BBS.QueryRequest
                {
                    TransactionId = transactionId
                };
                BBS.QueryResponse response = client.Query(Merchant, Token, request);
                BBS.PaymentInfo paymentInfo = (BBS.PaymentInfo)response;
                declineReason = string.Format("{0}/{1}: {2}", paymentInfo.Error.ResponseCode, paymentInfo.Error.ResponseSource, paymentInfo.Error.ResponseText);
            }

            order.GatewayResult = declineReason;
            order.TransactionStatus = "Decline";
            Services.Orders.Save(order);

            LogEvent(order, "Payment was declined: '{0}'", declineReason);
            return PrintErrorTemplate(order, string.Format("Payment was declined: '{0}'", declineReason));
        }

        #endregion

        /// <summary>
        /// Processes order payment
        /// </summary>
        /// <param name="order">Order which payment needs to be to be processed</param>
        /// <param name="transactionID">Id of transaction that is used for processing order</param>
        public void ProcessPaymentOperation(Order order, string transactionID)
        {
            try
            {
                var errorOccured = false;
                object paymentResult;
                if (Test)
                {
                    var clientTest = new BBSTest.NetaxeptClient();
                    BBSTest.ProcessRequest requestTest = new BBSTest.ProcessRequest
                    {
                        Operation = OperationType,
                        TransactionId = transactionID
                    };
                    BBSTest.ProcessResponse resultTest = clientTest.Process(Merchant, Token, requestTest);
                    paymentResult = resultTest;

                    var transactionRequest = new BBSTest.QueryRequest()
                    {
                        TransactionId = transactionID
                    };
                    BBSTest.PaymentInfo paymentInfoTest = clientTest.Query(Merchant, Token, transactionRequest) as BBSTest.PaymentInfo;
                    if (paymentInfoTest.Error != null && !string.IsNullOrEmpty(paymentInfoTest.Error.ResponseCode))
                    {
                        errorOccured = true;
                        LogEvent(order, "Order failed: {0}/{1}: {2}", paymentInfoTest.Error.ResponseCode, paymentInfoTest.Error.ResponseSource, paymentInfoTest.Error.ResponseText);
                    }
                    else
                    {
                        order.TransactionCardType = paymentInfoTest.CardInformation.PaymentMethod;
                        var maskedCard = paymentInfoTest.CardInformation.MaskedPAN;
                        var lastCardNumbers = maskedCard.Substring(maskedCard.Length - Math.Min(4, maskedCard.Length));
                        order.TransactionCardNumber = lastCardNumbers.PadLeft(16, 'X');
                        order.TransactionAmount = Converter.ToInt64(paymentInfoTest.OrderInformation.Amount) / 100;
                    }
                }
                else
                {
                    var client = new BBS.NetaxeptClient();
                    BBS.ProcessRequest request = new BBS.ProcessRequest
                    {
                        Operation = OperationType,
                        TransactionId = transactionID
                    };
                    BBS.ProcessResponse result = client.Process(Merchant, Token, request);
                    paymentResult = result;

                    var transactionRequest = new BBS.QueryRequest()
                    {
                        TransactionId = transactionID
                    };
                    BBS.PaymentInfo paymentInfo = client.Query(Merchant, Token, transactionRequest) as BBS.PaymentInfo;
                    if (paymentInfo.Error != null && !string.IsNullOrEmpty(paymentInfo.Error.ResponseCode))
                    {
                        errorOccured = true;
                        LogEvent(order, "Order failed: {0}/{1}: {2}", paymentInfo.Error.ResponseCode, paymentInfo.Error.ResponseSource, paymentInfo.Error.ResponseText);
                    }
                    else
                    {
                        order.TransactionCardType = paymentInfo.CardInformation.PaymentMethod;
                        var maskedCard = paymentInfo.CardInformation.MaskedPAN;
                        var lastCardNumbers = maskedCard.Substring(maskedCard.Length - Math.Min(4, maskedCard.Length));
                        order.TransactionCardNumber = lastCardNumbers.PadLeft(16, 'X');
                        order.TransactionAmount = Converter.ToInt64(paymentInfo.OrderInformation.Amount) / 100;
                    }
                }

                if (errorOccured)
                {
                    LogError(order, "At least one callback validation error exists - exiting callback routine. Transaction number: {0}", transactionID);

                    order.TransactionStatus = "Failed";
                }
                else
                {
                    LogEvent(order, "Payment succeeded with transaction number {0}", transactionID);

                    if (OperationType == "AUTH")
                    {
                        order.GatewayPaymentStatus = GatewayPaymentStatus.Accepted;
                    }
                    else if (OperationType == "SALE")
                    {
                        order.GatewayPaymentStatus = GatewayPaymentStatus.Paid;
                    }
                    else
                    {
                        order.GatewayPaymentStatus = GatewayPaymentStatus.Unknown;
                    }

                    order.TransactionStatus = "Succeeded";

                    SetOrderComplete(order, transactionID);
                }
                order.GatewayResult = SerializeRespnose(paymentResult);
                Services.Orders.UpdateGatewayResult(order, false);
            }
            finally
            {
                CheckoutDone(order);
            }

        }

        private static string SerializeRespnose(object paymentResult)
        {
            using (var str = new MemoryStream())
            {
                var xmlSerializer = new XmlSerializer(paymentResult.GetType());
                xmlSerializer.Serialize(str, paymentResult);
                str.Position = 0;
                using (var reader = new StreamReader(str))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        private string PrintErrorTemplate(Order order, string errMessage)
        {
            LogEvent(order, "Printing error template");

            var errorTemplate = new Template(ErrorTemplate);
            errorTemplate.SetTag("CheckoutHandler:ErrorMessage", errMessage);
            var orderRenderer = new Ecommerce.Frontend.Renderer();
            orderRenderer.RenderOrderDetails(errorTemplate, order, true);

            return errorTemplate.Output();
        }

        private Hashtable GetSupportedLanguages(bool addDefault)
        {
            var languages = new Hashtable {
                            { "no_NO", "Norwegian" },
                            { "sv_SE", "Swedish" },
                            { "da_DK", "Danish" },
                            { "fi_FI", "Finnish" },
                            { "en_GB", "English" },
                            { "de_DE", "German" },
                            { "fr_FR", "French" },
                            { "ru_RU", "Russian" },
                            { "pl_PL", "Polish" },
                            { "nl_NL", "Dutch" },
                            { "es_ES", "Spanish" },
                            { "it_IT", "Italian" },
                            { "pt_PT", "Portuguese" },
                            { "et_EE", "Estonian" },
                            { "lv_LV", "Latvian" },
                            { "lt_LT", "Lithuanian" }
                        };

            if (addDefault)
            {
                languages.Add("", "Default");
            }
            return languages;
        }

        private string GetLanguage()
        {
            var language = TerminalLanguage;
            if (string.IsNullOrEmpty(language))
            {
                var currentCulture = ExecutingContext.GetCulture(true);
                var languages = GetSupportedLanguages(false);
                if (languages.ContainsKey(currentCulture.Name))
                {
                    language = currentCulture.Name;
                }
            }
            return language;
        }

        #region IDropDownOptions

        /// <summary>
        /// Gets options according to behavior mode
        /// </summary>
        /// <param name="behaviorMode"></param>
        /// <returns>key-value pairs of settings</returns>
        public Hashtable GetOptions(string behaviorMode)
        {
            try
            {
                switch (behaviorMode)
                {
                    case "The type of operation":
                        return new Hashtable {
                            { "AUTH", Translate.Translate("Authorization only") },
                            { "SALE", Translate.Translate("Authorization and Capture") }
                        };
                    case "Terminal language":
                        return GetSupportedLanguages(true);

                    default:
                        throw new ArgumentException(string.Format("Unknown dropdown name: '{0}'", behaviorMode));
                }

            }
            catch (Exception ex)
            {
                LogError(null, ex, "Unhandled exception with message: {0}", ex.Message);
                return null;
            }
        }

        #endregion

        #region Implement IRemoteCapture

        /// <summary>
        /// Send capture request to transaction service
        /// </summary>
        /// <param name="order">Order to be captured</param>
        /// <returns>Response from transaction service</returns>
        public OrderCaptureInfo Capture(Order order)
        {
            try
            {
                if (order == null)
                {
                    LogError(null, "Order not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order not set");
                }
                if (string.IsNullOrEmpty(order.Id))
                {
                    LogError(null, "Order id not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order id not set");
                }
                if (string.IsNullOrEmpty(order.TransactionNumber))
                {
                    LogError(null, "Transaction number not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Transaction number not set");
                }
                if (order.Price.Price <= 0)
                {
                    LogError(null, "order amount not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "order amount not set");
                }

                if (Test)
                {
                    BBSTest.NetaxeptClient clientTest = new BBSTest.NetaxeptClient();
                    BBSTest.ProcessRequest requestTest = new BBSTest.ProcessRequest
                    {
                        Operation = "CAPTURE",
                        TransactionId = order.TransactionNumber,
                        TransactionAmount = order.Price.PricePIP.ToString()
                    };
                    clientTest.Process(Merchant, Token, requestTest);
                }
                else
                {
                    BBS.NetaxeptClient client = new BBS.NetaxeptClient();
                    BBS.ProcessRequest request = new BBS.ProcessRequest
                    {
                        Operation = "CAPTURE",
                        TransactionId = order.TransactionNumber,
                        TransactionAmount = order.Price.PricePIP.ToString()
                    };
                    client.Process(Merchant, Token, request);
                }

                LogEvent(order, "Capture successfull");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, "Capture successfull");
            }
            catch (Exception ex)
            {
                LogError(order, ex, "Unexpected error during capture: {0}", ex.Message);
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Unexpected error during capture");
            }
        }

        #endregion
    }
}
