﻿using Dynamicweb.Ecommerce.CheckoutHandlers.CyberSource.SimpleOrderApi;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.CyberSource
{
    /// <summary>
    /// Helper class that provides auxiliary methods for generating payment-specific data
    /// </summary>
    internal class PaymentHelper
    {
        /// <summary>
        /// Generates unique identifier for payment transaction
        /// </summary>
        /// <returns>string with GUID number</returns>
        internal static string GenerateTransactionUniqueID()
        {
            return Guid.NewGuid().ToString();
        }

        /// <summary>
        /// Returns current UTC date time stamp in following format: "yyyy-MM-dd'T'HH:mm:ss'Z'"
        /// </summary>
        /// <returns>DateTime stamp in UTC format</returns>
        public static string GetOperationTimeStamp()
        {
            return DateTime.Now.ToUniversalTime().ToString("yyyy-MM-dd'T'HH:mm:ss'Z'");
        }

        /// <summary>
        /// Builds newline-splited key-value pairs string representation
        /// </summary>
        /// <param name="keyValuePairs"></param>
        /// <returns>Multilune string representation of key and html encoded values pairs</returns>
        public static string BuildPostData(Dictionary<string, string> keyValuePairs)
        {
            return string.Join("\n", keyValuePairs.Select(x => string.Format("{0}={1}", x.Key, WebUtility.UrlEncode(x.Value))).ToArray());
        }

        /// <summary>
        /// Parses key-value pairs from input string using character delimeter
        /// </summary>
        /// <param name="inputString">String of key-value pairs</param>
        /// <returns></returns>
        public static NameValueCollection ParseResponse(string inputString)
        {
            char[] delimeter = { '=' };
            var result = new NameValueCollection();
            var stringReader = new StringReader(inputString);

            string line;
            while ((line = stringReader.ReadLine()) != null)
            {
                var parts = line.Split(delimeter, 2);
                if (parts.Length > 0)
                {
                    result.Add(parts[0], parts.Length == 2 ? parts[1] : null);
                }
            }

            return result;
        }

        /// <summary>
        /// Sets up CustomBinding parameters
        /// </summary>
        /// <returns>Custom binding object instance with asymmetric security binding, text binding and https transport data</returns>
        internal static CustomBinding GetSimpleOrderBinding()
        {
            var asymmetrycSecurityBindingElement = SecurityBindingElement.CreateUserNameOverTransportBindingElement();
            asymmetrycSecurityBindingElement.MessageSecurityVersion = MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10;
            asymmetrycSecurityBindingElement.IncludeTimestamp = false;
            asymmetrycSecurityBindingElement.SecurityHeaderLayout = SecurityHeaderLayout.Lax;

            var httpsTransport = new HttpsTransportBindingElement();
            httpsTransport.RequireClientCertificate = true;
            httpsTransport.AuthenticationScheme = AuthenticationSchemes.Anonymous;
            httpsTransport.MaxReceivedMessageSize = int.MaxValue;
            httpsTransport.UseDefaultWebProxy = false;

            var textBindingElement = new CustomTextMessageBindingElement();
            var currentBinding = new CustomBinding();
            currentBinding.Elements.Add(asymmetrycSecurityBindingElement);
            currentBinding.Elements.Add(textBindingElement);
            currentBinding.Elements.Add(httpsTransport);

            return currentBinding;
        }
    }
}
