using System;
using System.ServiceModel.Channels;
using System.Xml;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.CyberSource.SimpleOrderApi
{
    /// <summary>
    /// Custom binding element that specifies the character encoding and message versioning used for text-based SOAP messages
    /// </summary>
    public class CustomTextMessageBindingElement : MessageEncodingBindingElement
    {
        private XmlDictionaryReaderQuotas readerQuotas;

        /// <summary>
        /// Default constructor
        /// </summary>
        public CustomTextMessageBindingElement()
        {
            readerQuotas = new XmlDictionaryReaderQuotas();
        }

        #region IMessageEncodingBindingElement Members

        /// <summary>
        /// Constracts CustomTextMessageEncoderFactory instance
        /// </summary>
        /// <returns>new MessageEncoderFactory instance</returns>
        public override MessageEncoderFactory CreateMessageEncoderFactory()
        {
            return new CustomTextMessageEncoderFactory();
        }

        #endregion

        /// <summary>
        /// Returns message version
        /// </summary>
        public override MessageVersion MessageVersion
        {
            get { return MessageVersion.Soap12WSAddressing10; }
            set { }
        }

        /// <summary>
        /// Returns new CustomTextMessageEncoderFactory instance
        /// </summary>
        /// <returns>BindingElement instance</returns>
        public override BindingElement Clone()
        {
            return new CustomTextMessageBindingElement();
        }

        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }

        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            return context.CanBuildInnerChannelFactory<TChannel>();
        }

        public override T GetProperty<T>(BindingContext context)
        {
            if (typeof(T) == typeof(XmlDictionaryReaderQuotas))
            {
                return (T)(object)this.readerQuotas;
            }
            else
            {
                return base.GetProperty<T>(context);
            }
        }
    }
}
