﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.Cart;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.Gateways;
using Dynamicweb.Ecommerce.Products;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Rendering;
using System;
using System.Collections.Generic;
using System.Net;
using System.Xml;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.DocData
{
    /// <summary>
    /// DocData Checkout Handler
    /// </summary>
    [
		AddInName("DocData"),
		AddInDescription("Checkout handler for DocData")
	]
	public class DocData : CheckoutHandlerWithStatusPage
	{

		#region Addin parameters
		[AddInParameter("Merchant"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string Merchant { get; set; }

		[AddInParameter("Password"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string Password { get; set; }

		[AddInParameter("PaymentProfile"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string PaymentProfile { get; set; }

		[AddInParameter("DaysPayPeriod"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string DaysPayPeriod { get; set; }

		[AddInParameter("Language"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string Language { get; set; }

		[AddInParameter("Process Page"), AddInParameterEditor(typeof(TextParameterEditor), "")]
		public string ProcessPage { get; set; }

		[AddInParameter("Cancel template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/DocData/Cancel")]
		public string CancelTemplate { get; set; }

		[AddInParameter("Error template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/DocData/Error")]
		public string ErrorTemplate { get; set; }

		#endregion

		#region Form properties

		private static string BaseUrl(Order order)
		{
			var disablePortNumber = Dynamicweb.Configuration.SystemConfiguration.Instance.GetValue("/Globalsettings/System/http/DisableBaseHrefPort") == "True";
			var portString = (Context.Current.Request.Url.IsDefaultPort || disablePortNumber) ? string.Empty : string.Format(":{0}", Context.Current.Request.Url.Port);
			var pageId = Dynamicweb.Context.Current.Request["ID"] == null ? string.Empty : string.Format("ID={0}&", Dynamicweb.Context.Current.Request["ID"]);

			return string.Format("{2}://{0}{3}/Default.aspx?{1}{5}={4}", Context.Current.Request.Url.Host, pageId, Context.Current.Request.Url.Scheme, portString, order.Id, OrderIdRequestName);
		}

		private static string SuccessUrl(Order order)
		{
			return string.Format("{0}&DocDataState=Success", BaseUrl(order));
		}

		private static string CancelUrl(Order order)
		{
			return string.Format("{0}&DocDataState=Cancel", BaseUrl(order));
		}

		private static string ErrorUrl(Order order)
		{
			return string.Format("{0}&DocDataState=Error", BaseUrl(order));
		}

		private static string PendingUrl(Order order)
		{
			return string.Format("{0}&DocDataState=Pending", BaseUrl(order));
		}

		private string PrintErrorTemplate(Order order)
		{
			LogEvent(order, "Printing error template");

			var errorTemplate = new Template(ErrorTemplate);
			var orderRenderer = new Dynamicweb.Ecommerce.Frontend.Renderer();
			orderRenderer.RenderOrderDetails(errorTemplate, order, true);

			return errorTemplate.Output();
		}

		#endregion Form properties

		/// <summary>
		/// Initializes a new instance of the <see cref="DocData" /> class.
		/// </summary>
		public DocData()
		{
			HasServerCallback = true;
		}

		/// <summary>
		/// Starts order checkout procedurePost values to DocData
		/// </summary>
		/// <param name="order">Order to be checked out</param>
		/// <returns>String representation of template output</returns>
		public override string StartCheckout(Order order)
		{
			try
			{
				LogEvent(order, "Checkout started");

				string trkey = LoadGatewayTransactionID(order);

				var formValues = new Dictionary<string, string> {
					{ "command", "show_payment_cluster" },
					{ "merchant_name", Merchant },
					{ "merchant_transaction_id", order.Id },
					{ "client_language", Language },
					{ "return_url_success", SuccessUrl(order) },
					{ "return_url_canceled", CancelUrl(order) },
					{ "return_url_error", ErrorUrl(order) },
					{ "return_url_pending", PendingUrl(order) },
					{ "payment_cluster_key", trkey }
				};

				LogEvent(order, "Autopost to DocData");

				SubmitForm(ProcessPage, formValues);

				return string.Empty;
			}
			catch (System.Threading.ThreadAbortException)
			{
				return string.Empty;
			}
			catch (Exception ex)
			{
				LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
				return PrintErrorTemplate(order);
			}
		}

		#region LoadGatewayTransactionID
		private string LoadGatewayTransactionID(Order order)
		{
			string[] autoFields = {
				"merchant_name", "merchant_password", "merchant_transaction_id",
				"profile", "client_language", "cur_price",
				"price", "client_id", "client_email",
				"client_firstname", "client_lastname", "client_initials",
				"client_address", "client_zip", "client_city",
				"client_country", "days_pay_period"
			};
			string[] autoValues = {
				Merchant, Password, order.Id, PaymentProfile, Language, order.Price.Currency.Code,
				order.Price.Price.ToString("0.00", System.Globalization.CultureInfo.InvariantCulture),
				Converter.ToInt32(order.CustomerRefId).ToString(),
				EncodeString(order.CustomerEmail, 50),
				EncodeString(order.CustomerName, 50),
				EncodeString(order.CustomerName, 50),
				EncodeString(order.CustomerName, 50),
				EncodeString(order.CustomerAddress, 50),
				EncodeString(order.CustomerZip, 50),
				EncodeString(order.CustomerCity, 50),
				EncodeString(order.CustomerCountryCode, 50),
				DaysPayPeriod
			};

			string procPage = ProcessPage;

			string url = procPage + "?command=new_payment_cluster";
			for (int i = 0; i < autoFields.Length; i++)
			{
				url += "&" + autoFields[i] + "=" + autoValues[i];
			}

			var xmlDocument = new XmlDocument();
			xmlDocument.Load(url);
			string ret = xmlDocument.InnerXml;

			string transactionID = string.Empty;
			string transactionKey = string.Empty;
			try
			{
				transactionID = Converter.ToString(xmlDocument.SelectSingleNode("//*/id/@value").Value);
				transactionKey = Converter.ToString(xmlDocument.SelectSingleNode("//*/key/@value").Value);
			}
			catch (Exception ex)
			{
				order.Errors.Clear();
				var errorNodes = xmlDocument.SelectNodes("//*/error/@msg");
				foreach (XmlNode errorNode in errorNodes)
				{
					order.Errors.Add(errorNode.Value);
				}

				throw ex;
			}

			if (string.IsNullOrEmpty(transactionID) || string.IsNullOrEmpty(transactionKey))
			{
				throw new Exception("The payment cluster from DocData server is empty");
			}

			order.TransactionNumber = transactionKey;
			order.GatewayResult = PrepareGatewayResult(Dynamicweb.Context.Current.Request["ID"]);
			Services.Orders.UpdateGatewayResult(order, false);

			return transactionKey;
		}
		#endregion

		/// <summary>
		/// Redirects user to payment step or canceled order step
		/// </summary>
		/// <param name="order">Order for processing</param>
		/// <returns>String representation of template output</returns>
		public override string Redirect(Order order)
		{
			try
			{
				LogEvent(null, "Redirected to DocData CheckoutHandler");

				switch (Dynamicweb.Context.Current.Request["DocDataState"])
				{
					case "Success":
					case "Pending":
						return StateOk(order);

					case "Cancel":
					case "Error":
						return StateCancel(order);

					default:
						Callback(order);
						Context.Current.Response.End();
						return null;
				}
			}
			catch (System.Threading.ThreadAbortException)
			{
				return string.Empty;
			}
			catch (Exception ex)
			{
				LogError(order, ex, "Unhandled exception with message: {0}", ex.Message);
				return PrintErrorTemplate(order);
			}
		}

		private string StateOk(Order order)
		{
			LogEvent(order, "State ok");

			if (!order.Complete)
			{
				SetOrderComplete(order);
				CheckoutDone(order);
			}

			RedirectToCart(order);
			return null;
		}

		private string StateCancel(Order order)
		{
			LogEvent(order, "State cancel");

			order.TransactionStatus = "Cancelled";
			Services.Orders.Save(order);

			CheckoutDone(order);

			var cancelTemplate = new Template(CancelTemplate);
			var orderRenderer = new Dynamicweb.Ecommerce.Frontend.Renderer();
			orderRenderer.RenderOrderDetails(cancelTemplate, order, true);

			return cancelTemplate.Output();
		}

		private void Callback(Order order)
		{
			int pageID = 0;
			bool orderWasCompleted = order.Complete;

			try
			{
				LogEvent(order, "Callback started");

				string[] autoFieldsShow = { "merchant_name", "merchant_transaction_id", "merchant_password", "report_type" };
				string[] autoValuesShow = { Merchant, order.Id, Password, "xml_std" };

				string url = ProcessPage + "?command=status_payment_cluster";

				for (int i = 0; i < autoFieldsShow.Length; i++)
				{
					url += "&" + autoFieldsShow[i] + "=" + autoValuesShow[i];
				}

				url += "&payment_cluster_key=" + order.TransactionNumber;
				var webClient = new WebClient();
				var gatewayResult = webClient.DownloadString(url).Trim().Replace("\n", "").Replace("\r", "");

				var resultDocument = new XmlDocument();
				resultDocument.LoadXml(gatewayResult);

				var gatewayPaymentStatusResult = resultDocument.DocumentElement.SelectSingleNode("status/payment_cluster_process").ChildNodes[0].Value;
				order.GatewayPaymentStatus = ConvertDocDataStatusToDwStatus(gatewayPaymentStatusResult);

				if (order.GatewayPaymentStatus == GatewayPaymentStatus.Paid || order.GatewayPaymentStatus == GatewayPaymentStatus.ClosedSuccess)
				{
					LogEvent(order, "Payment {0} succeeded with transaction number {1}", order.Id, order.TransactionNumber);
					pageID = RetrievePageID(order.GatewayResult);

					SetOrderComplete(order);

					Services.Products.UpdateStock(order, ProductOrderStockOperator.Negative);
					order.GatewayTransactionProblems.Clear();
					order.GatewayResult = GetInfoFromQueryString(order);
					order.TransactionAmount = order.Price.Price;
					order.TransactionStatus = "Succeeded";
				}
				else
				{
					order.TransactionStatus = "Failed";
				}
				Services.Orders.SaveGatewayResult(order);
			}
			finally
			{
				if (!orderWasCompleted && order.Complete)
				{
					CheckoutDone(order, pageID);
				}
			}
		}

		private new void SetOrderComplete(Order order)
		{
			if (!order.Complete)
			{
				order.Complete = true;
				Services.Orders.Complete(order);
			}
		}

		private string ConvertDocDataStatusToDwStatus(string gatewayPaymentStatusResult)
		{
			switch (gatewayPaymentStatusResult)
			{
				case "paid":
					return GatewayPaymentStatus.Paid;
				case "chargedback":
					return GatewayPaymentStatus.ChargedBack;
				case "closed_success":
					return GatewayPaymentStatus.ClosedSuccess;
				case "closed_insufficientpaid":
					return GatewayPaymentStatus.ClosedInsufficientPaid;
				case "closed_expired":
					return GatewayPaymentStatus.ClosedExpired;
				case "closed_canceled":
					return GatewayPaymentStatus.ClosedCanceled;
				case "closed_refunded":
					return GatewayPaymentStatus.ClosedRefunded;
				case "hold":
					return GatewayPaymentStatus.Hold;
				case "new":
					return GatewayPaymentStatus.New;
				case "started":
					return GatewayPaymentStatus.Started;
				default:
					return GatewayPaymentStatus.Unknown;
			}
		}

		private string EncodeString(string value, int lenght)
		{
			if (string.IsNullOrEmpty(value))
			{
				return string.Empty;
			}

			value = WebUtility.UrlEncode(value.Trim());
			if (value.Length > lenght)
			{
				value = value.Substring(0, lenght);
			}

			return value;
		}
	}
}
