﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Net;
using System.Threading;
using Dynamicweb.Core;
using Dynamicweb.Core.Helpers;
using Dynamicweb.Ecommerce.Cart;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.Gateways;
using Dynamicweb.Ecommerce.Prices;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Rendering;
using Dynamicweb.SystemTools;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.EPayPaymentWindow
{
    /// <summary>
    /// ePay checkout handler
    /// </summary>
    [
        AddInName("ePay Payment Window"),
        AddInDescription("Checkout handler for ePay")
    ]
    public class EPayPaymentWindow : CheckoutHandler, IDropDownOptions, IRemoteCapture, IRemotePartialCapture
    {
        private static ConcurrentDictionary<string, Order> _currentOrders = new ConcurrentDictionary<string, Order>();
        private static ConcurrentDictionary<string, Order> _completedOrders = new ConcurrentDictionary<string, Order>();

        private enum PostModes { Auto, Template }
        private enum WindowStates { Overlay = 1, Fullscreen = 3 }

        private WindowStates windowstate = WindowStates.Overlay;
        private PostModes postMode;

        #region Addin parameters

        [AddInParameter("Merchant"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Merchant { get; set; }

        [AddInParameter("Window ID"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string WindowID { get; set; }

        [AddInParameter("Group"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Group { get; set; }

        [AddInParameter("Auth Mail"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string AuthMail { get; set; }

        [AddInParameter("Auth SMS"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string AuthSMS { get; set; }

        [AddInParameter("Instantcapture"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public string Instantcapture { get; set; }

        [AddInParameter("Splitpayment"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public string Splitpayment { get; set; }

        [AddInParameter("MD5 Key"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string MD5Key { get; set; }

        [AddInParameter("Encoding"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Encoding { get; set; }

        [AddInParameter("CSS URL"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string CssUrl { get; set; }

        [AddInParameter("Mobile CSS URL"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string MobileCssUrl { get; set; }

        [AddInParameter("Order text"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string OrderText { get; set; }

        [AddInParameter("Description"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string Description { get; set; }

        [AddInParameter("Google tracker"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string GoogleTracker { get; set; }

        [AddInParameter("Background color"), AddInParameterEditor(typeof(TextParameterEditor), "infoText=HEX color code without #")]
        public string BackgroundColor
        {
            get
            {
                if (string.IsNullOrWhiteSpace(_backgroundColor) || !System.Text.RegularExpressions.Regex.IsMatch(_backgroundColor, @"\A\b[0-9a-fA-F]{3,6}\b\Z"))
                {
                    return "A7A7A7";
                }
                return _backgroundColor;
            }
            set
            {
                if (value.StartsWith("#"))
                {
                    value = value.TrimStart(new char[] { '#' });
                }
                _backgroundColor = value;
            }
        }
        private string _backgroundColor = "A7A7A7";

        [AddInParameter("Opacity"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Opacity { get; set; }

        [AddInParameter("Decline text"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string DeclineText { get; set; }

        [AddInParameter("Window state"), AddInParameterEditor(typeof(DropDownParameterEditor), "none=false; SortBy=Value;")]
        public string WindowStateSelection
        {
            get
            {
                return windowstate.ToString();
            }
            set
            {
                switch (value)
                {
                    case "Overlay":
                        windowstate = WindowStates.Overlay;
                        break;
                    case "Fullscreen":
                        windowstate = WindowStates.Fullscreen;
                        break;
                }
            }
        }

        [AddInParameter("Post mode"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true; none=false; SortBy=Value;")]
        public string PostModeSelection
        {
            get
            {
                return postMode.ToString();
            }
            set
            {
                switch (value)
                {
                    case "Auto":
                        postMode = PostModes.Auto;
                        break;

                    case "Template":
                        postMode = PostModes.Template;
                        break;
                }
            }
        }

        [AddInParameter("Post template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/ePay/Post")]
        public string PostTemplate { get; set; }

        [AddInParameter("Cancel template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/ePay/Cancel")]
        public string CancelTemplate { get; set; }

        [AddInParameter("Error template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/ePay/Error")]
        public string ErrorTemplate { get; set; }

        [AddInParameter("Card fee added by ePay"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool UseAddFee { get; set; }

        [AddInParameter("Payment timeout"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public int TimeOut { get; set; }

        [AddInParameter("Add Mobilepay to payment options"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool AddMobilepayAsPaymentOption { get; set; }

        [AddInParameter("Use Mobilepay as only payment option"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool UseMobilepayAsOnlyPaymentOption { get; set; }

        #endregion

        #region Form properties

        private static string LanguageCode
        {
            get
            {
                var currentLanguageCode = Dynamicweb.Environment.ExecutingContext.GetCulture(true).TwoLetterISOLanguageName;
                var supportedLanguageCodes = new List<string>(new[] { "da", "en", "se", "no", "gl", "is", "de", "fi", "es", "fr", "pl", "it", "nl" });
                return supportedLanguageCodes.Contains(currentLanguageCode) ? currentLanguageCode : "en";
            }
        }

        private static int GetLanguageID(string languageCode)
        {
            switch (languageCode)
            {
                case "da":
                    return 1;
                case "en":
                    return 2;
                case "se":
                    return 3;
                case "no":
                    return 4;
                case "gl":
                    return 5;
                case "is":
                    return 6;
                case "de":
                    return 7;
                case "fi":
                    return 8;
                case "es":
                    return 9;
                case "fr":
                    return 10;
                case "pl":
                    return 11;
                case "it":
                    return 12;
                case "nl":
                    return 13;
                default:
                    return 0;
            }
        }

        private static string BaseUrl(Order order)
        {
            var portString = Context.Current.Request.Url.IsDefaultPort ? string.Empty : string.Format(":{0}", Context.Current.Request.Url.Port);
            var pageId = Dynamicweb.Context.Current.Request["ID"] == null ? string.Empty : string.Format("ID={0}&", Dynamicweb.Context.Current.Request["ID"]);

            return string.Format("{2}://{0}{3}/Default.aspx?{1}{5}={4}", Context.Current.Request.Url.Host, pageId, Context.Current.Request.Url.Scheme, portString, order.Id, OrderIdRequestName);
        }

        private static string AcceptUrl(Order order)
        {
            return string.Format("{0}&ePayState=Ok", BaseUrl(order));
        }

        private static string DeclineUrl(Order order)
        {
            return string.Format("{0}&ePayState=Cancel", BaseUrl(order));
        }

        private static string CallbackUrl(Order order)
        {
            return string.Format("{0}&ePayState=Callback&redirect=false", BaseUrl(order));
        }

        private string BoolToInt(string boolValue)
        {
            if (boolValue == "True")
            {
                return "1";
            }
            else
            {
                return "0";
            }
        }

        #endregion

        #region Constructors


        /// <summary>
        /// Initializes a new instance of the <see cref="EPayPaymentWindow" /> class.
        /// </summary>
        public EPayPaymentWindow()
        {
            ServicePointManager.SecurityProtocol = ServicePointManager.SecurityProtocol | SecurityProtocolType.Tls12;

            WindowID = "1";
            postMode = PostModes.Auto;
            Encoding = "UTF-8";
            Opacity = "50";
        }

        #endregion

        #region Process checkout

        /// <summary>
		/// Starts order checkout procedure
        /// </summary>
		/// <param name="order">Order to be checked out</param>
		/// <returns>String representation of template output</returns>
        public override string StartCheckout(Order order)
        {
            try
            {
                LogEvent(order, "Checkout started");
                var formValues = new Dictionary<string, string> {
                    { "merchantnumber", Merchant },
                    { "windowid", WindowID },
                    { "language", GetLanguageID(LanguageCode).ToString() },
                    { "orderid", order.Id },
                    { "amount", order.Price.PricePIP.ToString() },
                    { "currency", order.Price.Currency.PayGatewayCode.ToString() },
                    { "accepturl", AcceptUrl(order) },
                    { "cancelurl", DeclineUrl(order) },
                    { "callbackurl", CallbackUrl(order) },
                    { "smsreceipt", AuthSMS },
                    { "mailreceipt", AuthMail },
                    { "instantcallback", "1" },
                    { "group", Group },
                    { "windowstate", Convert.ToInt32(windowstate).ToString() },
                    { "cms", "dynamicweb" },
                    { "ownreceipt", "1" },
                    { "timeout", Convert.ToInt32(TimeOut).ToString() },
                    { "instantcapture", BoolToInt(Instantcapture) },
                    { "splitpayment", BoolToInt(Splitpayment) },
                    { "encoding", Encoding },
                    { "ordertext", OrderText },
                    { "description", Description },
                    { "googletracker", GoogleTracker },
                    { "backgroundcolor", BackgroundColor },
                    { "opacity", Opacity },
                    { "declinetext", DeclineText },
                };

                if (isValidUrl(CssUrl))
                {
                    formValues.Add("cssurl", CssUrl);
                }
                if (isValidUrl(MobileCssUrl))
                {
                    formValues.Add("mobilecssurl", MobileCssUrl);
                }

                if (AddMobilepayAsPaymentOption)
                {
                    formValues.Add("paymentcollection", "4");
                }

                if (UseMobilepayAsOnlyPaymentOption)
                {
                    formValues.Add("paymenttype", "29");
                }

                formValues.Add("hash", StringHelper.Md5HashToString(string.Join("", formValues.Values) + MD5Key));

                switch (postMode)
                {
                    case PostModes.Auto:
                        LogEvent(order, "Auto post to ePay PaymentWindow");

                        SubmitForm("https://ssl.ditonlinebetalingssystem.dk/integration/ewindow/Default.aspx", formValues);

                        return string.Empty;

                    case PostModes.Template:
                        LogEvent(order, "Render template");

                        var formTemplate = new Template(PostTemplate);
                        foreach (var formValue in formValues)
                        {
                            formTemplate.SetTag(string.Format("ePay.{0}", formValue.Key), formValue.Value);
                        }

                        return formTemplate.Output();

                    default:
                        LogError(order, "Unhandled post mode: '{0}'", postMode);
                        throw new ArgumentException(string.Format("Unhandled post mode: '{0}'", postMode));
                }
            }
            catch (ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order);
            }
        }

        private bool isValidUrl(string url)
        {
            Uri uri;
            return Uri.TryCreate(url, UriKind.Absolute, out uri) && (uri.Scheme == Uri.UriSchemeHttp || uri.Scheme == Uri.UriSchemeHttps);
        }

        /// <summary>
        /// Redirects user to payment step or canceled order step
        /// </summary>
        /// <param name="order">Order for processing</param>
        /// <returns>String representation of template output</returns>
        public override string Redirect(Order order)
        {
            try
            {
                LogEvent(null, "Redirected to ePay CheckoutHandler");

                var ePayStateValue = Context.Current.Request["ePayState"];
                if (ePayStateValue == null)
                {
                    LogError(null, "The request variable 'ePayState' is not present");
                    return "";
                }

                switch (Converter.ToString(ePayStateValue))
                {
                    case "Ok":
                        return StateOk(order);

                    case "Cancel":
                        return StateCancel(order);

                    case "Callback":
                        Callback(order);
                        Context.Current.Response.End();
                        return null;

                    default:
                        LogError(order, "Unknown ePayState: '{0}'", Converter.ToString(Context.Current.Request["ePayState"]));
                        return PrintErrorTemplate(order);
                }
            }
            catch (ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order);
            }
        }

        private string StateOk(Order order)
        {
            LogEvent(order, "State ok");

            order = StartProcessingOrder(order);
            var oldOrderId = order.Id;

            if (!order.Complete)
            {
                ProcessOrder(order);
            }

            StopProcessingOrder(oldOrderId);

            // Redirect to cart
            RedirectToCart(order);
            return null;
        }

        private string StateCancel(Order order)
        {
            order = StartProcessingOrder(order);
            var oldOrderId = order.Id;

            LogEvent(order, "State cancel");

            order.TransactionStatus = "Cancelled";
            Services.Orders.Save(order);

            CheckoutDone(order);

            StopProcessingOrder(oldOrderId);

            var cancelTemplate = new Template(CancelTemplate);
            return Render(order, cancelTemplate);
        }

        private void Callback(Order order)
        {
            LogEvent(order, "Callback started - checking order");

            order = StartProcessingOrder(order);
            var oldOrderId = order.Id;

            LogEvent(order, "Callback started - order ok");

            if (!order.Complete)
            {
                ProcessOrder(order);
            }

            StopProcessingOrder(oldOrderId);
        }

        #region "Order processing"

        private void ProcessOrder(Order order)
        {
            try
            {
                var parameters = Context.Current.Request.QueryString;
                var calculatedhash = string.Empty;

                foreach (string parameter in parameters)
                {
                    if (parameter != "hash")
                    {
                        calculatedhash += parameters[parameter];
                    }
                }
                calculatedhash += MD5Key;

                if (StringHelper.Md5HashToString(calculatedhash) != parameters["hash"])
                {
                    LogError(order, "The MD5 checksum returned from callback does not match: Callback: {0}, calculated: {1}", parameters["hash"], StringHelper.Md5HashToString(calculatedhash));
                    order.TransactionStatus = "Failed";
                    Services.Orders.Save(order);
                }
                else if (!string.IsNullOrEmpty(parameters["error"]))
                {
                    LogError(order, "The ePay server returns error code: {0}. Error codes are available in your Bambora administration.", parameters["error"]);
                    order.TransactionStatus = "Failed";
                    Services.Orders.Save(order);
                }
                else if (string.IsNullOrEmpty(parameters["txnid"]))
                {
                    LogError(order, "No transaction number sent to callback.");
                    order.TransactionStatus = "Failed";
                    Services.Orders.Save(order);
                }
                else
                {
                    LogEvent(order, "Payment succeeded with transaction number {0}", parameters["txnid"]);

                    if (UseAddFee)
                    {
                        LogEvent(order, "Checking card fee");
                        var fee = parameters["txnfee"];
                        long feeAmount;
                        if (long.TryParse(fee, out feeAmount))
                        {
                            LogEvent(order, "Saving card fee '{0}'", fee);
                            order.ExternalPaymentFee = feeAmount / 100d;
                        }
                    }

                    order.TransactionAmount = order.Price.PricePIP / 100d;
                    order.TransactionStatus = "Succeeded";
                    int paytypeid = Converter.ToInt32(parameters["paymenttype"]);
                    string paytype;
                    paymentTypes().TryGetValue(paytypeid, out paytype);

                    order.TransactionCardType = paytype;
                    order.TransactionCardNumber = Converter.ToString(parameters["cardno"]);

                    if (Instantcapture == "True")
                    {
                        string infoTxt = "Capture succeeded";
                        LogEvent(order, infoTxt);
                        order.CaptureInfo = new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, infoTxt);
                    }

                    SetOrderComplete(order, parameters["txnid"]);
                }
            }
            finally
            {
                CheckoutDone(order);
            }
        }

        #endregion

        private Order StartProcessingOrder(Order order)
        {
            Order newOrder = null;
            if (_currentOrders.ContainsKey(order.Id))
            {
                var startTime = DateTime.Now;
                do
                {
                    if (DateTime.Now.Subtract(startTime).TotalSeconds > 120)
                        break;

                    if (newOrder == null)
                        _currentOrders.TryGetValue(order.Id, out newOrder);

                    if (_currentOrders.ContainsKey(order.Id))
                        Thread.Sleep(500);
                } while (_currentOrders.ContainsKey(order.Id));
                return newOrder;
            }
            else if (_completedOrders.ContainsKey(order.Id))
            {
                _completedOrders.TryGetValue(order.Id, out newOrder);
                return newOrder;
            }
            else
            {
                _currentOrders.AddOrUpdate(order.Id, order, (key, existingValue) => { return order; });
                return order;
            }
        }

        private void StopProcessingOrder(string orderId)
        {
            Order outOrder;

            _currentOrders.TryRemove(orderId, out outOrder);

            if (_completedOrders.ContainsKey(orderId))
                _completedOrders.TryRemove(orderId, out outOrder);
            else if (outOrder != null && outOrder.Complete)
                _completedOrders.AddOrUpdate(orderId, outOrder, (key, existingOrder) => { return outOrder; });
        }

        private string PrintErrorTemplate(Order order)
        {
            LogEvent(order, "Printing error template");
            var errorTemplate = new Template(ErrorTemplate);

            return Render(order, errorTemplate);
        }

        #endregion

        #region IDropDownOptions

        /// <summary>
        /// Gets options according to behavior mode
        /// </summary>
        /// <param name="behaviorMode"></param>
        /// <returns>Key-value pairs of settings</returns>
        public Hashtable GetOptions(string behaviorMode)
        {
            try
            {
                switch (behaviorMode)
                {
                    case "Post mode":
                        return new Hashtable {
                        { "Auto", Translate.Translate("Auto post (does not use the template)") },
                        { "Template", Translate.Translate("Render template") }
                                   };
                    case "Window state":
                        return new Hashtable {
                        { "Overlay", Translate.Translate("Overlay (simple integration only)") },
                        { "Fullscreen", Translate.Translate("Full screen") }
                                   };
                    default:
                        throw new ArgumentException(string.Format("Unknown dropdown name: '{0}'", behaviorMode));
                }
            }
            catch (Exception ex)
            {
                LogError(null, "Unhandled exception with message: {0}", ex.Message);
                return null;
            }
        }
        #endregion

        #region IRemoteCapture

        /// <summary>
        /// Send capture request to transaction service
        /// </summary>
        /// <param name="order">Order to be captured</param>
        /// <returns><see cref="OrderCaptureInfo">Order Cupture Info object</see> represents capture information about order</returns>
        public OrderCaptureInfo Capture(Order order)
        {
            return Capture(order, order.Price.PricePIP + PriceHelper.ConvertToPIP(order.Currency, order.ExternalPaymentFee), false);
        }

        /// <summary>
        /// Send capture request to transaction service
        /// </summary>
        /// <param name="order">Order to be captured</param>
        /// <param name="amount">Amount to be transferred. The amount must be given in smallest possible unit, e.g. cents 100 = USD 1.</param>
        /// <param name="final">Shows that this step is final</param>
        /// <returns><see cref="OrderCaptureInfo">Order Cupture Info object</see> represents capture information about order</returns>
        public OrderCaptureInfo Capture(Order order, long amount, bool final)
        {
            try
            {
                if (order == null)
                {
                    LogError(null, "Order not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order not set");
                }
                if (string.IsNullOrEmpty(order.Id))
                {
                    LogError(null, "Order id not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Order id not set");
                }
                if (order.Price.PricePIP < amount)
                {
                    LogError(null, "Amount to capture should be less of order total");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Amount to capture should be less of order total");
                }
                if (string.IsNullOrEmpty(order.TransactionNumber))
                {
                    LogError(null, "Transaction number not set");
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Transaction number not set");
                }

                //var amount = order.Price.PricePIP + PriceInfo.ConvertToPIP(order.ExternalPaymentFee);
                var authXml =
                "<?xml version=\"1.0\" encoding=\"" + Encoding + "\"?>\n" +
                "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                "  <soap:Body>\n" +
                "    <capture xmlns=\"https://ssl.ditonlinebetalingssystem.dk/remote/payment\">\n" +
                "      <merchantnumber>" + Merchant + "</merchantnumber>\n" +
                "      <transactionid>" + order.TransactionNumber + "</transactionid>" +
                "      <amount>" + amount + "</amount>" +
                "    </capture>\n" +
                "  </soap:Body>\n" +
                "</soap:Envelope>";

                var wc = new WebClient();
                string strResponse = null;
                byte[] response = null;
                byte[] byteArray = null;

                byteArray = System.Text.Encoding.ASCII.GetBytes(authXml);
                wc.Headers.Add("Content-type", "text/xml; charset=UTF-8");
                wc.Headers.Add("SOAPAction", "https://ssl.ditonlinebetalingssystem.dk/remote/payment/capture");
                response = wc.UploadData("https://ssl.ditonlinebetalingssystem.dk/remote/payment.asmx", byteArray);
                strResponse = System.Text.Encoding.GetEncoding(Encoding).GetString(response);

                if (GetNodeText("captureResult", strResponse) == "false")
                {
                    LogError(order, "pbsResponse and epayresponse received from ePay: pbsResponse: {0}, epayresponse: {1}", GetNodeText("pbsResponse", strResponse), GetNodeText("epayresponse", strResponse));
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Incorrect pbsResponse and epayresponse code received from ePay");
                }

                if (final)
                {
                    LogEvent(order, string.Format("Message=\"{0}\" Amount=\"{1:f2}\"", "Split capture (final)", amount / 100f), DebuggingInfoType.CaptureResult);
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, "Split capture successful - final");
                }
                else if (amount < (order.Price.PricePIP + PriceHelper.ConvertToPIP(order.Currency, order.ExternalPaymentFee)))
                {
                    LogEvent(order, string.Format("Message=\"{0}\" Amount=\"{1:f2}\"", "Split capture", amount / 100f), DebuggingInfoType.CaptureResult);
                    return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Split, "Split capture successful");
                }
                LogEvent(order, "Capture successfull");
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Success, "Capture successfull");

            }
            catch (Exception ex)
            {
                LogError(order, "Unexpected error during capture: {0}", ex.Message);
                return new OrderCaptureInfo(OrderCaptureInfo.OrderCaptureState.Failed, "Unexpected error during capture");
            }
        }

        /// <summary>
		/// Shows if capture supported
		/// </summary>
		/// <param name="order">This order object</param>
		/// <returns>'true' if order transaction card type is defined</returns>
        public bool CaptureSupported(Order order)
        {
            return true;
        }

        /// <summary>
        /// Shows if partial capture of the order supported
        /// </summary>
        /// <param name="order">Instance of order</param>
        /// <returns>True, if partial capture of the order is supported</returns>
        public bool SplitCaptureSupported(Order order)
        {
            return true;
        }

        private string GetNodeText(string nodeName, string xmlDocumentText)
        {
            string result = null;
            int startPosition, endPosition = 0;
            var start = "<" + nodeName + ">";
            var end = "</" + nodeName + ">";

            if ((startPosition = xmlDocumentText.IndexOf(start)) > 0 && (endPosition = xmlDocumentText.IndexOf(end)) > 0 && endPosition > startPosition)
            {
                result = xmlDocumentText.Substring(startPosition + start.Length, endPosition - (startPosition + start.Length));
            }

            return result;
        }

        private Dictionary<int, string> paymentTypes()
        {
            var types = new Dictionary<int, string>();
            types.Add(1, "Dankort / Visa Dankort");
            types.Add(3, "Visa / Visa Electron");
            types.Add(4, "Mastercard");
            types.Add(6, "JCB");
            types.Add(7, "Maestro");
            types.Add(8, "Diners Club");
            types.Add(9, "American Express");
            types.Add(11, "Forbrugsforeningen");
            types.Add(13, "Danske Netbetalinger");
            types.Add(14, "PayPal");
            types.Add(17, "Klarna");
            types.Add(18, "SveaWebPay");
            types.Add(19, "SEB(SE)");
            types.Add(20, "Nordea(SE)");
            types.Add(21, "Handelsbanken(SE)");
            types.Add(22, "Swedbank(SE)");
            types.Add(23, "ViaBill");
            types.Add(24, "Beeptify");
            types.Add(25, "iDEAL");
            types.Add(27, "Paii");
            types.Add(28, "Brandts Gavekort");
            types.Add(29, "MobilePay Online");
            types.Add(31, "Ekspress Bank");
            types.Add(34, "Masterpass");

            return types;
        }

        #endregion
    }
}
