﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.Cart;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.Gateways;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Rendering;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace Dynamicweb.Ecommerce.CheckoutHandlers.Ogone
{
    /// <summary>
    /// Ogone Checkout Handler
    /// </summary>
    [
        AddInName("Ingenico ePayments"),
        AddInDescription("Checkout handler for Ingenico ePayments")
    ]
    public class Ogone : CheckoutHandlerWithStatusPage, IDropDownOptions
    {
        #region SHA IN/OUT fields

        private readonly List<string> FieldsIn = new List<string> {
            "AMOUNT", "ORDERID", "PSPID",
            "CURRENCY", "CN", "EMAIL",
            "LANGUAGE", "TITLE", "BGCOLOR",
            "TXTCOLOR", "TBLBGCOLOR", "TBLTXTCOLOR",
            "BUTTONBGCOLOR", "BUTTONTXTCOLOR", "LOGO",
            "FONTTYPE", "TP", "PM",
            "COM", "BRAND", "OWNERZIP",
            "OWNERADDRESS", "ACCEPTURL", "DECLINEURL",
            "EXCEPTIONURL", "CANCELURL", "PARAMPLUS"
        };

        private static readonly string[] FieldsOut = {
            "AAVADDRESS",
            "AAVCHECK",
            "AAVZIP",
            "ACCEPTANCE",
            "ALIAS",
            "AMOUNT",
            "BIN",
            "BRAND",
            "CARDNO",
            "CCCTY",
            "CN",
            "COMPLUS",
            "CREATION_STATUS",
            "CURRENCY",
            "CVCCHECK",
            "DCC_COMMPERCENTAGE",
            "DCC_CONVAMOUNT",
            "DCC_CONVCCY",
            "DCC_EXCHRATE",
            "DCC_EXCHRATESOURCE",
            "DCC_EXCHRATETS",
            "DCC_INDICATOR",
            "DCC_MARGINPERCENTAGE",
            "DCC_VALIDHOURS",
            "DIGESTCARDNO",
            "ECI",
            "ED",
            "ENCCARDNO",
            "IP",
            "IPCTY",
            "NBREMAILUSAGE",
            "NBRIPUSAGE",
            "NBRIPUSAGE_ALLTX",
            "NBRUSAGE",
            "NCERROR",
            "ORDERID",
            "PAYID",
            "PM",
            "SCO_CATEGORY",
            "SCORING",
            "STATUS",
            "SUBSCRIPTION_ID",
            "TRXDATE",
            "VC"
        };
        #endregion

        #region Addin parameters

        [AddInParameter("Merchant"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Merchant { get; set; }

        [AddInParameter("Process page"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string ProcessPage { get; set; }

        [AddInParameter("Secret key"), AddInParameterEditor(typeof(TextParameterEditor), "size=80")]
        public string SecretKey { get; set; }

        [AddInParameter("Referer"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string Referer { get; set; }

        [AddInParameter("IPRanges"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string IPRanges { get; set; }

        [AddInParameter("SHA sign syntax"), AddInParameterEditor(typeof(RadioParameterEditor), "")]
        public string SHASignSyntax { get; set; }

        [AddInParameter("Post template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/Ogone/Post")]
        public string PostTemplate { get; set; }

        [AddInParameter("Cancel template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/Ogone/Cancel")]
        public string CancelTemplate { get; set; }

        [AddInParameter("Error template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=templates/eCom7/CheckoutHandler/Ogone/Error")]
        public string ErrorTemplate { get; set; }

        [AddInParameter("Afterpay Integration"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool IsAfterpayIntegration { get; set; }

        [AddInParameter("Social security number / Company registration number"), AddInParameterEditor(typeof(TextParameterEditor), "")]
        public string CompanyNumber { get; set; }

        #endregion  Addin parameters

        #region Form properties
        private string Sha1(string strPassword)
        {
            var dwPassword = new SHA1CryptoServiceProvider();
            byte[] hashedDataBytes = null;
            var encoder = new UTF8Encoding();

            if (strPassword == null)
            {
                strPassword = string.Empty;
            }
            hashedDataBytes = dwPassword.ComputeHash(encoder.GetBytes(strPassword));

            var stringHashPassword = new StringBuilder();

            foreach (byte obPassword in hashedDataBytes)
            {
                stringHashPassword.Append(obPassword.ToString("x2"));
            }

            return stringHashPassword.ToString();
        }

        private string MakeSha1ForSignIn(Order order, Dictionary<string, string> autoFields)
        {
            string hash = string.Empty;

            if (SHASignSyntax == "EachFollowed")
            {
                foreach (string key in FieldsIn.OrderBy(s => s))
                {
                    string fieldValue = autoFields.ContainsKey(key) ? autoFields[key] : Dynamicweb.Context.Current.Request[key];
                    if (!string.IsNullOrEmpty(fieldValue))
                    {
                        hash += string.Format("{0}={1}{2}", key, fieldValue, SecretKey);
                    }
                }
            }
            else
            {
                hash = order.Id + order.Price.PricePIP + order.Price.Currency.Code + Merchant + SecretKey;
            }

            return Sha1(hash);
        }

        private string MakeSha1ForSignOut()
        {
            string hash = string.Empty;

            if (SHASignSyntax == "EachFollowed")
            {
                foreach (string key in FieldsOut.OrderBy(s => s))
                {
                    string val = Dynamicweb.Context.Current.Request[key];
                    if (!string.IsNullOrEmpty(val))
                    {
                        hash += string.Format("{0}={1}{2}", key, val, SecretKey);
                    }
                }
            }
            else
            {
                hash = string.Format("{0}{1}{2}{3}{4}{5}{6}{7}{8}{9}{10}",
                    Dynamicweb.Context.Current.Request["orderID"],
                    Dynamicweb.Context.Current.Request["currency"],
                    Dynamicweb.Context.Current.Request["amount"],
                    Dynamicweb.Context.Current.Request["PM"],
                    Dynamicweb.Context.Current.Request["acceptance"],
                    Dynamicweb.Context.Current.Request["STATUS"],
                    Dynamicweb.Context.Current.Request["CARDNO"],
                    Dynamicweb.Context.Current.Request["PAYID"],
                    Dynamicweb.Context.Current.Request["NCERROR"],
                    Dynamicweb.Context.Current.Request["BRAND"],
                    SecretKey
                );
            }

            return Sha1(hash);
        }

        private static string BaseUrl(Order order)
        {
            var disablePortNumber = Dynamicweb.Configuration.SystemConfiguration.Instance.GetValue("/Globalsettings/System/http/DisableBaseHrefPort") == "True";
            var portString = (Context.Current.Request.Url.IsDefaultPort || disablePortNumber) ? "" : string.Format(":{0}", Context.Current.Request.Url.Port);
            var pageId = Dynamicweb.Context.Current.Request["ID"] == null ? string.Empty : string.Format("ID={0}&", Dynamicweb.Context.Current.Request["ID"]);

            return string.Format("{2}://{0}{3}/Default.aspx?{1}{5}={4}", Context.Current.Request.Url.Host, pageId, Context.Current.Request.Url.Scheme, portString, order.Id, OrderIdRequestName);
        }

        private static string ContinueUrl(Order order)
        {
            return string.Format("{0}&OgoneState=Ok", BaseUrl(order));
        }

        private static string CancelUrl(Order order)
        {
            return string.Format("{0}&OgoneState=Cancel", BaseUrl(order));
        }

        private string PrintErrorTemplate(Order order)
        {
            LogEvent(order, "Printing error template");
            var errorTemplate = new Template(ErrorTemplate);
            var orderRenderer = new Dynamicweb.Ecommerce.Frontend.Renderer();
            orderRenderer.RenderOrderDetails(errorTemplate, order, true);

            return errorTemplate.Output();
        }

        #endregion Form properties

        /// <summary>
        /// Initializes a new instance of the <see cref="Ogone" /> class.
        /// </summary>
        public Ogone()
        {
            ServicePointManager.SecurityProtocol = ServicePointManager.SecurityProtocol | SecurityProtocolType.Tls12;
            HasServerCallback = true;
            SHASignSyntax = "EachFollowed";
        }

        /// <summary>
        /// Starts order checkout procedure
        /// </summary>
        /// <param name="order">Order to be checked out</param>
        /// <returns>String representation of template output</returns>
        public override string StartCheckout(Order order)
        {
            try
            {
                LogEvent(order, "Checkout started");
                LogEvent(order, "Render Ogone template");

                var formTemplate = new Template(PostTemplate);
                formTemplate.SetTag("Ogone.OrderID", order.Id);
                formTemplate.SetTag("Ogone.Order.Price.PricePIP", order.Price.PricePIP);
                formTemplate.SetTag("Ogone.Order.Price.PriceWithVATFormatted", order.Price.PriceWithVATFormatted);

                formTemplate.SetTag("Ogone.State", "Payment");
                formTemplate.SetTag("Ogone.continueurl", ContinueUrl(order));
                formTemplate.SetTag("Ogone.cancelurl", CancelUrl(order));
                formTemplate.SetTag("Ogone.pageid", Dynamicweb.Context.Current.Request["ID"]);

                return formTemplate.Output();
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order);
            }
        }

        /// <summary>
        /// Redirects user to Ogone Checkout step
        /// </summary>
        /// <param name="order">Order for processing</param>
        /// <returns>String representation of template output</returns>
        public override string Redirect(Order order)
        {
            try
            {
                LogEvent(order, "Redirected to Ogone CheckoutHandler");

                switch (Converter.ToString(Context.Current.Request["OgoneState"]))
                {
                    case "Payment":
                        if (!string.IsNullOrEmpty(Converter.ToString(Context.Current.Request["OgonePricePIP"])))
                        {
                            long postedPip = Converter.ToInt64(Context.Current.Request["OgonePricePIP"]);
                            if (order.Price.PricePIP != postedPip)
                            {
                                LogEvent(order, $"Price mismatch detected. Order price PIP is {order.Price.PricePIP} - got this value from the cart post {postedPip}");
                                return StateCancel(order);
                            }
                        }

                        return StatePayment(order);

                    case "Ok":
                        return StateOk(order);

                    case "Cancel":
                        return StateCancel(order);

                    default:
                        Callback(order);
                        return null;
                }
            }
            catch (System.Threading.ThreadAbortException)
            {
                return string.Empty;
            }
            catch (Exception ex)
            {
                LogError(order, "Unhandled exception with message: {0}", ex.Message);
                return PrintErrorTemplate(order);
            }
        }

        private string StatePayment(Order order)
        {
            LogEvent(order, $"State payment - PIP amount {order.Price.Currency.Code} {order.Price.PricePIP} / {order.Price.PriceWithVATFormattedNoSymbol}");

            Dictionary<string, string> autoFields = new Dictionary<string, string> {
                { "PSPID", Merchant },
                { "ORDERID", order.Id },
                { "AMOUNT", order.Price.PricePIP.ToString() },
                { "CURRENCY", order.Price.Currency.Code },
                { "CN", order.CustomerName },
                { "EMAIL", order.CustomerEmail }
            };

            if (IsAfterpayIntegration)
            {
                AfterpayIntegration(order, autoFields);
            }

            var parameters = string.Format("{0}={1}", "SHASign", WebUtility.UrlEncode(MakeSha1ForSignIn(order, autoFields)));
            foreach (string key in FieldsIn)
            {
                if (autoFields.ContainsKey(key))
                {
                    parameters += string.Format("&{0}={1}", key, WebUtility.UrlEncode(autoFields[key]));
                }
                else
                {
                    parameters += string.Format("&{0}={1}", key, WebUtility.UrlEncode(Dynamicweb.Context.Current.Request[key]));
                }
            }

            string ret = string.Empty;
            using (var webClient = new WebClient())
            {
                webClient.Headers[HttpRequestHeader.Referer] = Referer;

                if (IsAfterpayIntegration)
                {
                    webClient.Headers[HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
                    ret = webClient.UploadString(ProcessPage, parameters);
                }
                else
                {
                    ret = webClient.DownloadString(string.Format("{0}?{1}", ProcessPage, parameters));
                }
            }

            int absolutePathLength = ProcessPage.LastIndexOf('/');
            if (absolutePathLength > 0)
            {
                string absolutePath = ProcessPage.Substring(0, absolutePathLength);
                ret = Regex.Replace(ret, "(<script.*?src=[\"'])(?!http)(.+?)([\"'].*?>)", String.Format("$1{0}/$2$3", absolutePath), RegexOptions.IgnoreCase | RegexOptions.Multiline);
            }

            var match = Regex.Match(ret, "name=\"payid\"\\s+value=\"([^\"]*)\"", RegexOptions.IgnoreCase);

            string transactionID = string.Empty;
            if (match.Success)
            {
                transactionID = Converter.ToString(match.Groups[1].Value);
            }
            if (transactionID.Length == 0 || Converter.ToInt32(transactionID) <= 0)
            {
                LogError(order, "Missing transaction ID from Ogone - DATA: " + ret.ToString());
            }

            order.TransactionNumber = transactionID;
            order.GatewayResult = PrepareGatewayResult(Dynamicweb.Context.Current.Request["pageID"]);
            Services.Orders.UpdateGatewayResult(order, false);

            Context.Current.Response.Write(ret);
            Context.Current.Response.End();
            return null;
        }

        private void AfterpayIntegration(Order order, IDictionary<string, string> autoFields)
        {
            var afterpayFields = new Dictionary<string, string>();
            afterpayFields.Add("CUID", CompanyNumber);
            afterpayFields.Add("CIVILITY", Truncate(Dynamicweb.Context.Current.Request["CIVILITY"], 10));

            string firstName, lastName;
            if (string.IsNullOrEmpty(order.CustomerSurname))
            {
                var customerName = Converter.ToString(order.CustomerName);
                var delPos = customerName.IndexOf(' ');
                firstName = delPos >= 0 ? customerName.Substring(0, delPos) : customerName;
                lastName = delPos >= 0 ? customerName.Substring(delPos + 1) : string.Empty;
            }
            else
            {
                firstName = order.CustomerName;
                lastName = order.CustomerSurname;
            }
            afterpayFields.Add("ECOM_BILLTO_POSTAL_NAME_FIRST", firstName);
            afterpayFields.Add("ECOM_BILLTO_POSTAL_NAME_LAST", lastName);

            afterpayFields.Add("OWNERADDRESS", order.CustomerAddress);
            if (!string.IsNullOrEmpty(order.CustomerHouseNumber))
            {
                afterpayFields.Add("ECOM_BILLTO_POSTAL_STREET_NUMBER", order.CustomerHouseNumber);
            }
            else
            {
                afterpayFields.Add("ECOM_BILLTO_POSTAL_STREET_NUMBER", order.CustomerAddress2);
            }
            afterpayFields.Add("OWNERZIP", order.CustomerZip);
            afterpayFields.Add("OWNERTOWN", order.CustomerCity);
            afterpayFields.Add("OWNERCTY", order.CustomerCountryCode);
            afterpayFields.Add("OWNERTELNO", string.IsNullOrEmpty(order.CustomerCell) ? order.CustomerPhone : order.CustomerCell);

            var isDeliveryFieldsFilled = !string.IsNullOrEmpty(order.DeliveryZip);
            afterpayFields.Add("ECOM_SHIPTO_POSTAL_NAME_PREFIX", Truncate(Dynamicweb.Context.Current.Request["ECOM_SHIPTO_POSTAL_NAME_PREFIX"], 10));
            afterpayFields.Add("ECOM_SHIPTO_DOB", Truncate(Dynamicweb.Context.Current.Request["ECOM_SHIPTO_DOB"], 10));

            if (isDeliveryFieldsFilled)
            {
                if (string.IsNullOrEmpty(order.DeliverySurname))
                {
                    string deliveryName = Converter.ToString(order.DeliveryName);
                    int delPos = deliveryName.IndexOf(' ');
                    firstName = delPos >= 0 ? deliveryName.Substring(0, delPos) : deliveryName;
                    lastName = delPos >= 0 ? deliveryName.Substring(delPos + 1) : "";
                }
                else
                {
                    firstName = order.DeliveryName;
                    lastName = order.DeliverySurname;
                }
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_NAME_FIRST", firstName);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_NAME_LAST", lastName);

                afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_LINE1", order.DeliveryAddress);
                if (!string.IsNullOrEmpty(order.DeliveryHouseNumber))
                {
                    afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_NUMBER", order.DeliveryHouseNumber);
                }
                else
                {
                    afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_NUMBER", order.DeliveryAddress2);
                }
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_POSTALCODE", order.DeliveryZip);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_CITY", order.DeliveryCity);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_COUNTRYCODE", order.DeliveryCountryCode);
                afterpayFields.Add("ECOM_SHIPTO_ONLINE_EMAIL", order.DeliveryEmail);
            }
            else
            {
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_NAME_FIRST", firstName);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_NAME_LAST", lastName);

                afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_LINE1", order.CustomerAddress);
                if (!string.IsNullOrEmpty(order.CustomerHouseNumber))
                {
                    afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_NUMBER", order.CustomerHouseNumber);
                }
                else
                {
                    afterpayFields.Add("ECOM_SHIPTO_POSTAL_STREET_NUMBER", order.CustomerAddress2);
                }
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_POSTALCODE", order.CustomerZip);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_CITY", order.CustomerCity);
                afterpayFields.Add("ECOM_SHIPTO_POSTAL_COUNTRYCODE", order.CustomerCountryCode);
                afterpayFields.Add("ECOM_SHIPTO_ONLINE_EMAIL", order.CustomerEmail);
            }

            int index = 0;
            foreach (var orderLine in order.OrderLines)
            {
                index++;
                afterpayFields.Add(string.Format("ITEMID{0}", index), orderLine.Id);
                afterpayFields.Add(string.Format("ITEMNAME{0}", index), orderLine.ProductName);
                afterpayFields.Add(string.Format("ITEMPRICE{0}", index), orderLine.UnitPrice.PriceWithVAT.ToString("0.00", System.Globalization.CultureInfo.InvariantCulture));
                afterpayFields.Add(string.Format("ITEMQUANT{0}", index), orderLine.Quantity.ToString(System.Globalization.CultureInfo.InvariantCulture));
                afterpayFields.Add(string.Format("ITEMVATCODE{0}", index), orderLine.Price.VATPercent.ToString(System.Globalization.CultureInfo.InvariantCulture) + "%");
                afterpayFields.Add(string.Format("TAXINCLUDED{0}", index), "1");
            }

            var shipping = order.ShippingFee;
            if (shipping.PriceWithVAT != 0)
            {
                index++;
                afterpayFields.Add(string.Format("ITEMID{0}", index), "SHIPMENTFEE");
                afterpayFields.Add(string.Format("ITEMNAME{0}", index), Dynamicweb.SystemTools.Translate.Translate("SHIPMENT FEE"));
                afterpayFields.Add(string.Format("ITEMPRICE{0}", index), shipping.PriceWithVAT.ToString("0.00", System.Globalization.CultureInfo.InvariantCulture));
                afterpayFields.Add(string.Format("ITEMQUANT{0}", index), "1");
                afterpayFields.Add(string.Format("ITEMVATCODE{0}", index), shipping.VATPercent.ToString(System.Globalization.CultureInfo.InvariantCulture) + "%");
                afterpayFields.Add(string.Format("TAXINCLUDED{0}", index), "1");
            }

            var payment = order.PaymentFee;
            if (payment.PriceWithVAT != 0)
            {
                index++;
                afterpayFields.Add(string.Format("ITEMID{0}", index), "PAYMENTFEE");
                afterpayFields.Add(string.Format("ITEMNAME{0}", index), Dynamicweb.SystemTools.Translate.Translate("PAYMENT FEE"));
                afterpayFields.Add(string.Format("ITEMPRICE{0}", index), payment.PriceWithVAT.ToString("0.00", System.Globalization.CultureInfo.InvariantCulture));
                afterpayFields.Add(string.Format("ITEMQUANT{0}", index), "1");
                afterpayFields.Add(string.Format("ITEMVATCODE{0}", index), payment.VATPercent.ToString(System.Globalization.CultureInfo.InvariantCulture) + "%");
                afterpayFields.Add(string.Format("TAXINCLUDED{0}", index), "1");
            }

            foreach (var af in afterpayFields)
            {
                if (!FieldsIn.Contains(af.Key)) FieldsIn.Add(af.Key);
                if (!autoFields.ContainsKey(af.Key)) autoFields.Add(af.Key, af.Value);
            }
        }

        private static string Truncate(string source, int length)
        {
            if (string.IsNullOrEmpty(source))
            {
                return string.Empty;
            }

            return (source.Length > length) ? source.Substring(0, length) : source;
        }

        private string StateOk(Order order)
        {
            LogEvent(order, "State ok");

            try
            {
                SetOrderComplete(order);
            }
            finally
            {
                CheckoutDone(order);
            }

            RedirectToCart(order);
            return null;

        }

        private string StateCancel(Order order)
        {
            LogEvent(order, "State cancel");

            order.TransactionStatus = "Cancelled";
            Services.Orders.Save(order);

            CheckoutDone(order);

            var cancelTemplate = new Template(CancelTemplate);
            var orderRenderer = new Dynamicweb.Ecommerce.Frontend.Renderer();
            orderRenderer.RenderOrderDetails(cancelTemplate, order, true);

            return cancelTemplate.Output();
        }

        private void Callback(Order order)
        {
            int pageID = 0;
            bool orderWasCompleted = order.Complete;
            bool wasError = false;

            try
            {
                string transactionStatus = Converter.ToString(Dynamicweb.Context.Current.Request["STATUS"]);
                LogEvent(order, "Callback started. Payment status: {0}", transactionStatus);

                if (Dynamicweb.Context.Current.Request["PAYID"] != order.TransactionNumber)
                {
                    LogError(
                        order,
                        "The ordernumber returned from callback does not match with the ordernumber set on the order: Callback: '{0}', order: '{1}'",
                        Dynamicweb.Context.Current.Request["PAYID"], order.TransactionNumber
                    );
                    wasError = true;
                }

                var sha1Sign = MakeSha1ForSignOut();
                if (!sha1Sign.Equals(Dynamicweb.Context.Current.Request["SHASIGN"], StringComparison.CurrentCultureIgnoreCase))
                {
                    LogError(
                        order,
                        "The Sha1 checksum returned from callback does not match: Callback: {0}, calculated: {1}",
                        Dynamicweb.Context.Current.Request["SHASIGN"], sha1Sign
                    );
                    wasError = true;
                }

                if (wasError)
                {
                    LogError(order, "At least one callback validation error exists - exiting callback routine.");
                }
                else
                {
                    order.GatewayPaymentStatus = ConvertOgoneStatusToDwStatus(transactionStatus);
                    if (order.GatewayPaymentStatus == GatewayPaymentStatus.Paid || order.GatewayPaymentStatus == GatewayPaymentStatus.ClosedSuccess)
                    {
                        LogEvent(order, "Payment {0} succeeded with transaction number {1}", order.Id, order.TransactionNumber);
                        pageID = RetrievePageID(order.GatewayResult);

                        SetOrderComplete(order);

                        order.GatewayTransactionProblems.Clear();
                        order.GatewayResult = GetInfoFromQueryString(order);
                    }
                }

                if (order.Complete)
                {
                    order.TransactionAmount = order.Price.Price;
                }

                order.TransactionStatus = transactionStatus;
                Services.Orders.SaveGatewayResult(order);
            }
            finally
            {
                if (!orderWasCompleted && order.Complete)
                {
                    CheckoutDone(order, pageID);
                }
            }
        }

        private string ConvertOgoneStatusToDwStatus(string gatewayPaymentStatusResult)
        {
            switch (gatewayPaymentStatusResult.Trim())
            {
                case "0": // Incomplete or invalid
                    return GatewayPaymentStatus.Declined;
                case "1": // Cancelled by client
                    return GatewayPaymentStatus.Rejected;
                case "2": // Authorization refused
                    return GatewayPaymentStatus.Declined;
                case "4": // Order stored
                    return GatewayPaymentStatus.Hold;
                case "41": // Order stored
                    return GatewayPaymentStatus.Hold;
                case "5": // Authorized
                    return GatewayPaymentStatus.Accepted;
                case "51": // Authorization waiting
                    return GatewayPaymentStatus.Accepted;
                case "6": // Authorized and cancelled
                    return GatewayPaymentStatus.Rejected;
                case "7": // Payment deleted
                    return GatewayPaymentStatus.Rejected;
                case "9": // Payment(requested)
                    return GatewayPaymentStatus.Paid;
                case "91": // Payment processing
                    return GatewayPaymentStatus.Paid;
                default:
                    return GatewayPaymentStatus.Unknown;
            }
        }

        /// <summary>
        /// Gets options according to behavior mode
        /// </summary>
        /// <param name="behaviorMode"></param>
        /// <returns>Key-value pairs of settings</returns>
        public Hashtable GetOptions(string behaviorMode)
        {
            try
            {
                switch (behaviorMode)
                {
                    case "SHA sign syntax":
                        return new Hashtable {
                        { "MainOnly", Dynamicweb.SystemTools.Translate.Translate("Main parameters only (used before December 2011)") },
                        { "EachFollowed", Dynamicweb.SystemTools.Translate.Translate("Each parameter followed by passphrase") }
                    };
                    default:
                        throw new ArgumentException(string.Format("Unknown dropdown name: '{0}'", behaviorMode));
                }
            }
            catch (Exception ex)
            {
                LogError(null, "Unhandled exception with message: {0}", ex.Message);
                return null;
            }
        }
    }
}
